#include <stdio.h>
#include <stdlib.h>
#include <png.h>
#include "mpegcodec.h"

static char *AUTHORS[]={
    "Ruben Lopez Gomez",
    NULL
};

void MPEGcodec::haveFileName() {
    MPEG_VideoInfo info;
    MPEG_SystemInfo sinfo;
    mpeg = new MPEG(filename,false);
    mpeg->GetVideoInfo(&info);
    mpeg->GetSystemInfo(&sinfo);
    surf=SDL_CreateRGBSurface(SDL_SWSURFACE,
			      info.width, info.height, 32,
			      0x000000FF,0x0000FF00,0x00FF0000,0);
    width=info.width;
    height=info.height;
    memset(surf->pixels, 0xFF, width*height*4);

    if (surf==NULL) {
	printf("Error creating surface\n");
        return;
    }
    mutex=SDL_CreateMutex();
    if (! mpeg->SetDisplay(surf, mutex, NULL) ) {
	printf("ERROR setting MPEG display :-m\n");
        return;
    }
    // Calculating number of frames (this is uuuugly ;)
    mpeg->Skip(sinfo.total_time+1.0);
    mpeg->GetVideoInfo(&info);
    _size=info.current_frame+1;
    mpeg->Seek(0);
    printf("NOTICE: MPEG of %d frames, %f seconds\n",_size,sinfo.total_time);
    if (_size==0) _size=1;
}

char** MPEGcodec::getLoaderAuthors() {
    return AUTHORS;
}

Image * MPEGcodec::read(int position) {
    if (mpeg==NULL) {
	printf("MPEGcodec ERROR: I've not filename\n");
        return NULL;
    }
    mpeg->RenderFrame(position);
    SDL_LockMutex(mutex);
    SDL_LockSurface(surf);
    Image *im=new Image(width,height);
    memcpy(im->frameBuffer(),surf->pixels,width*height*4);
    Pixel *pix=im->frameBuffer();
    for (int i=0;i<width*height;i++) {pix->a=0xFF;pix++;}
    SDL_UnlockSurface(surf);
    SDL_UnlockMutex(mutex);
    return im;
}

#define END1 ".mpeg"
#define END2 ".mpg"

bool MPEGcodec::canLoad(const char *filename) {
    return (
	    (strstr(filename,END1) == filename + strlen(filename) - strlen(END1))
	 || (strstr(filename,END2) == filename + strlen(filename) - strlen(END2))
	   );
}


// Instaling this input codec automatically when dinamic loaded.
extern "C" {

    class proxy {
    public:
	proxy() {
	    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)<0) {
		fprintf(stderr, "MPEGcodec ERROR: Couldn't init SDL video: %s\n",
			SDL_GetError());

	    }
	    register_video_loader(new MPEGcodec());
	}
    };

    proxy filter;
}
