#include <stdio.h>
#include <stdlib.h>
#include <png.h>
#include "pngcodec.h"

static Image *readpng(const char *file) {
    unsigned int y;
    png_uint_32 width, height;
    int bit_depth, color_type, interlace_type;
    png_structp png_ptr=NULL;
    png_infop info_ptr;
    FILE *fp=fopen(file,"rb");
    if (fp==NULL) {
	//error
	return NULL;
    }
    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
				     NULL,NULL,NULL);
    if (png_ptr == NULL) {
	fclose(fp);
	//error
	return NULL;
    }

    info_ptr = png_create_info_struct(png_ptr);
    if (info_ptr == NULL)
    {
	fclose(fp);
	png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
	//error
	return NULL;
    }

    if (setjmp(png_ptr->jmpbuf))
    {
	/* Free all of the memory associated with the png_ptr and info_ptr */
	png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
	fclose(fp);
	/* If we get here, we had a problem reading the file */
	//error
	return NULL;
    }

    png_init_io(png_ptr, fp);
    
    png_read_info(png_ptr, info_ptr);

    png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type,
		 &interlace_type, NULL, NULL);

    /* Expand paletted colors into true RGB triplets */
    if (color_type == PNG_COLOR_TYPE_PALETTE)
	png_set_expand(png_ptr);
    
    /* Expand paletted or RGB images with transparency to full alpha channels
     * so the data will be available as RGBA quartets.
     */
    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
	png_set_expand(png_ptr);

    /* Add filler byte (before/after each RGB triplet) */
    if ( (color_type&PNG_COLOR_MASK_ALPHA) == 0)
	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);

    png_read_update_info(png_ptr, info_ptr);
    
    Image *im=new Image(width,height);

    png_bytep *row_pointers=new (unsigned char*)[height];
    Pixel *framebuffer=im->frameBuffer();
    
    for (y=0;y<height;y++) {
	row_pointers[y]=(png_bytep)framebuffer+y*width*sizeof(Pixel);
    }

    png_read_image(png_ptr, row_pointers);

    /* read rest of file, and get additional chunks in info_ptr - REQUIRED */
    png_read_end(png_ptr, info_ptr);

    /* clean up after the read, and free any memory allocated - REQUIRED */
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);

    
    fclose(fp);
    delete(row_pointers);
    return im;
}

static char *AUTHORS[]={
    "Ruben Lopez Gomez",
    NULL
};

char** PNG::getLoaderAuthors() {
    return AUTHORS;
}

Image * PNG::read(int position) {
    return readpng(filename);
}

#define END ".png"

bool PNG::canLoad(const char *filename) {
    return (strstr(filename,END) == filename+strlen(filename)-strlen(END));
}


// Instaling this input codec automatically when dinamic loaded.
extern "C" {

    class proxy {
    public:
	proxy() {
	    register_video_loader(new PNG());
	}
    };

    proxy filter;
}
