#include <stdlib.h>
#include "common.h"

/* The system uses a variety of data files.  By opening them via this
   function, we can accommodate various locations. */

FILE *OpenTableFile(char *name)
{
  char fulname[80];
  char *envdir;
  FILE *f;

  fulname[0] = '\0';

#ifdef TABLES_PATH
  strcpy(fulname, TABLES_PATH);   /* default relative path for tables */
#endif /* TABLES_PATH */          /* (includes terminal path seperator */

#ifdef UNIX                       /* envir. variables for UNIX only */
  {
    /*    char *getenv(); */

    envdir = getenv(MPEGTABENV);   /* check for environment */
    if(envdir != NULL)
      strcpy(fulname, envdir);
    strcat(fulname, PATH_SEPARATOR);  /* add a "/" on the end */
  }
#endif /* UNIX */

  strcat(fulname, name);
  if( (f=fopen(fulname,"r"))==NULL )
    {
      fprintf(stderr,"OpenTable: could not find %s\n", fulname);

#ifdef UNIX
      if(envdir != NULL)
        fprintf(stderr,"Check %s directory '%s'\n",MPEGTABENV, envdir);
      else
        fprintf(stderr,"Check local directory './%s' or setenv %s\n",
                TABLES_PATH, MPEGTABENV);
#else /* not unix : no environment variables */

#ifdef TABLES_PATH
      fprintf(stderr,"Check local directory './%s'\n",TABLES_PATH);
#endif /* TABLES_PATH */

#endif /* UNIX */

    }
  return f;
}

/***********************************************************************
*
* Read one of the data files ("alloc_*") specifying the bit allocation/
* quatization parameters for each subband in layer II encoding
*
**********************************************************************/

int read_bit_alloc(int table, al_table *alloc)
/* read in table, return # subbands */
{
  unsigned int a, b, c, d, i, j;
  FILE *fp;
  char name[16], t[80];
  int sblim;

  strcpy(name, "alloc_0");

  switch (table)
    {
    case 0 :
      name[6] = '0';
      break;
    case 1 :
      name[6] = '1';
      break;
    case 2 :
      name[6] = '2';
      break;
    case 3 :
      name[6] = '3';
      break;
    case 4 :
      name[6] = '4';
      break; /* LSF, added 1995-07-11 shn */
    default :
      name[6] = '0';
    }

  if (!(fp = OpenTableFile(name)))
    {
      printf("Please check bit allocation table %s\n", name);
      exit(1);
    }

  printf("using bit allocation table %s\n", name);

  fgets(t, 80, fp);
  sscanf(t, "%d\n", &sblim);
  while (!feof(fp))
    {
      fgets(t, 80, fp);
      sscanf(t, "%d %d %d %d %d %d\n", &i, &j, &a, &b, &c, &d);
      (*alloc)[i][j].steps = a;
      (*alloc)[i][j].bits  = b;
      (*alloc)[i][j].group = c;
      (*alloc)[i][j].quant = d;
    }
  fclose(fp);
  return sblim;
}

/***********************************************************************
*
* Using the decoded info the appropriate possible quantization per
* subband table is loaded
*
**********************************************************************/

int pick_table(frame_params *fr_ps)
/* choose table, load if necess, return # sb's */
{
  int table, lay, ws, bsp, br_per_ch, sfrq;
  int sblim = fr_ps->sblimit;     /* return current value if no load */

  lay = fr_ps->header->lay - 1;
  bsp = fr_ps->header->bitrate_index;
  br_per_ch = bitrate[fr_ps->header->version][bsp] / fr_ps->stereo;
  ws = fr_ps->header->sampling_frequency;
  sfrq = s_freq[fr_ps->header->version][ws];
  /* decision rules refer to per-channel bitrates (kbits/sec/chan) */
  if (fr_ps->header->version == MPEG_AUDIO_ID)
    { /* MPEG-1 */
      if ((sfrq == 48 && br_per_ch >= 56) ||
          (br_per_ch >= 56 && br_per_ch <= 80)) table = 0;
      else if (sfrq != 48 && br_per_ch >= 96) table = 1;
      else if (sfrq != 32 && br_per_ch <= 48) table = 2;
      else table = 3;
    }
  else
    { /* MPEG-2 LSF */
      table = 4;
    }
  if (fr_ps->tab_num != table)
    {
      if (fr_ps->tab_num >= 0)
        mem_free((void **)&(fr_ps->alloc));
      fr_ps->alloc = (al_table FAR *) mem_alloc(sizeof(al_table),
                     "alloc");
      sblim = read_bit_alloc(fr_ps->tab_num = table, fr_ps->alloc);
    }
  return sblim;
}
