// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include <stdlib.h>
#include <videocodec.h>

#include <config.h>

#include <mpeg.h>
#include <prototypes.h>
#include <search.h>
#include <param.h>
#include <opts.h>
#include <fsize.h>
#include <specifics.h>
#include <frame.h>

// Global variables :( needed by ucbmpeg
extern "C" {
    extern time_t IOtime;
    int	whichGOP = -1;
    boolean	childProcess = FALSE;
    boolean	ioServer = FALSE;
    boolean	outputServer = FALSE;
    boolean	decodeServer = FALSE;
    int	quietTime = 0;
    boolean realQuiet = FALSE;
    boolean	frameSummary = TRUE;
    boolean debugSockets = FALSE;
    boolean debugMachines = FALSE;
    boolean showBitRatePerFrame = FALSE;
    boolean	computeMVHist = FALSE;
    int     baseFormat;
    extern  boolean specificsOn;
    extern  FrameSpecList *fsl;
    boolean pureDCT=FALSE;
    char    encoder_name[1024];
} // extern "C"

static ImStream *thestream;
static Image *buffer;
static int build_position=0;
static int build_size=0;

#define NAME "UCBMPEG MPEG-1 codec";
#define VERSION "0.5"
#define COPYRIGHT "Copyright (c) 1995 The Regents of the University of California."

char *authors[]={
    "ucbmpeg people",
    "Ruben Lopez <ryu@mundivia.es>",
    NULL
};

#define COMMENTS \
"This codec allow you to write standard MPEG-1 files \
with a lot of options and very fast. See setup for \
more information on the options supported by this codec."

class UcbMpeg: public VideoCodec {
public:
    UcbMpeg();
    void doEncode (ImStream *stream, char *filename, Image *backbuffer);
    char *getName();
    char *getVersion();
    char *getCopyright();
    char **getAuthors();
    char *getDescription();
    bool configure();
    bool saveConfig(char *directory);
    void progress(double val) { notifyProgress(val); }
};

static UcbMpeg *thecodec=NULL;

UcbMpeg::UcbMpeg() {
    // Defaults
    numInputFiles = 0;
    numMachines = 0;
    SetFramePattern("I");
    SetGOPSize(30);
    SetSlicesPerFrame(1);
    SetIQScale(8);
    SetPQScale(10);
    SetBQScale(25);
    SetPixelSearch("FULL");
    SetSearchRange(10,10);
    SetPSearchAlg("LOGARITHMIC");
    SetBSearchAlg("SIMPLE");
    SetReferenceFrameType("ORIGINAL");
    thecodec=this;
}

char* UcbMpeg::getName() {
    return NAME;
}

char *UcbMpeg::getVersion() {
    return VERSION;
}

char *UcbMpeg::getCopyright() {
    return COPYRIGHT;
}

char **UcbMpeg::getAuthors() {
    return authors;
}

char *UcbMpeg::getDescription() {
    return COMMENTS;
}

void UcbMpeg::doEncode(ImStream *stream, char *filename, Image *backbuff) {
    numInputFiles=stream->size();
    int totalTime;
    thestream=stream;
    FILE * outfile=fopen(filename,"wb");
    if (!outfile) {
        printf("ERROR creating output file\n");
    }

    buffer=backbuff;
    realWidth=buffer->width();
    realHeight=buffer->height();
    yuvWidth=realWidth;
    yuvHeight=realHeight;

    // Compress
    Tune_Init();
    Frame_Init();
    if (specificsOn) Specifics_Init();
    ComputeFrameTable();

    build_size=stream->size();
    build_position=1;
    totalTime = GenMPEGStream(whichGOP, -1, numInputFiles-1,
			      customQtable, customNIQtable,
			      numInputFiles, outfile,
			      filename);
}

bool UcbMpeg::configure() {
    new Config();
    return false;
}

bool UcbMpeg::saveConfig(char *directory) {
    /* Not implemented yet */
    return false;
}

extern "C" {

boolean
ReadNextImage(MpegFrame *mf)
{

    int y;
    Fsize_Note(mf->id, realWidth, realHeight);

    mf->rgb_maxval = 255;

    Frame_AllocPPM(mf);

    thestream->nextImage(buffer);
    
    for ( y = 0; y < realHeight; y++ ) {
	int x;
	for (x=0;x<realWidth;x++) {
            buffer->getPixel(x,y,mf->ppm_data[y]+x*3,mf->ppm_data[y]+1+x*3,mf->ppm_data[y]+2+x*3);
	}
    }
    thecodec->progress((double)build_position++/(double)build_size);
    return TRUE;
}

    
}

// Instaling this codec automatically when dinamic loaded.
extern "C" {

    class proxy {
    public:
	proxy() {
	    register_video_codec(new UcbMpeg());
	}
    };

    proxy p;
}
