// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <streameditor.h>
#include "filter.h"
#include <gui_visitor.h>

class SampleEditor: public StreamEditor {
    int end;
    void setValues();
public:
    void show();
    void notice(char *event, Subject *source);
    void action(char *action);
    Prototype *clone();
    void setStart(int i);
    void setEnd(int i);
    void setExtend(bool x);
};


/* Callbacks generated by glade, filled by me */
gboolean
on_sample_filter_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        SampleEditor    *sf)
{
    delete sf;
    return TRUE;
}


void
on_start_changed                       (GtkEditable     *editable,
                                        SampleEditor    *sf)
{
    char *s=gtk_entry_get_text(GTK_ENTRY(editable));
    char *end;
    int i=strtol(s,&end,10);
    if (end!=s) {
        sf->setStart(i);
    }
}


void
on_end_changed                         (GtkEditable     *editable,
                                        SampleEditor    *sf)
{
    char *s=gtk_entry_get_text(GTK_ENTRY(editable));
    char *end;
    int i=strtol(s,&end,10);
    if (end!=s) {
        sf->setEnd(i);
    }
}


void
on_extend_toggled                      (GtkToggleButton *togglebutton,
                                        SampleEditor    *sf)
{
    sf->setExtend(togglebutton->active == 1);
}


void
on_sense_clicked                       (GtkButton       *button,
                                        SampleEditor    *sf)
{
    sf->action("SENSE");
}


void
on_edit_clicked                        (GtkButton       *button,
                                        SampleEditor    *sf)
{
    sf->action("EDIT");
}


void
on_close_clicked                       (GtkButton       *button,
                                        SampleEditor    *sf)
{
    delete sf;
}


/* Function generated by glade */
GtkWidget*
create_sample_filter (SampleEditor *v)
{
  GtkWidget *sample_filter;
  GtkWidget *vbox26;
  GtkWidget *frame57;
  GtkWidget *vbox27;
  GtkWidget *alignment6;
  GtkWidget *label45;
  GtkWidget *table9;
  GtkObject *start_adj;
  GtkWidget *start;
  GtkObject *end_adj;
  GtkWidget *end;
  GtkWidget *label44;
  GtkWidget *label43;
  GtkWidget *extend;
  GtkWidget *hbox33;
  GtkWidget *label46;
  GtkWidget *sense;
  GtkWidget *sense_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *hbox32;
  GtkWidget *frame58;
  GtkWidget *edit;
  GtkWidget *close;

  sample_filter = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (sample_filter), "sample_filter", sample_filter);
  gtk_window_set_title (GTK_WINDOW (sample_filter), _("Sample Filter Editor"));

  vbox26 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox26);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "vbox26", vbox26,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox26);
  gtk_container_add (GTK_CONTAINER (sample_filter), vbox26);
  gtk_container_set_border_width (GTK_CONTAINER (vbox26), 10);

  frame57 = gtk_frame_new (_("Limits"));
  gtk_widget_ref (frame57);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "frame57", frame57,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame57);
  gtk_box_pack_start (GTK_BOX (vbox26), frame57, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame57), 5);

  vbox27 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox27);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "vbox27", vbox27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox27);
  gtk_container_add (GTK_CONTAINER (frame57), vbox27);

  alignment6 = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_ref (alignment6);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "alignment6", alignment6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment6);
  gtk_box_pack_start (GTK_BOX (vbox27), alignment6, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (alignment6), 10);

  label45 = gtk_label_new (_("Select Here the limits for the effect."));
  gtk_widget_ref (label45);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "label45", label45,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label45);
  gtk_container_add (GTK_CONTAINER (alignment6), label45);
  gtk_label_set_justify (GTK_LABEL (label45), GTK_JUSTIFY_LEFT);

  table9 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table9);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "table9", table9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table9);
  gtk_box_pack_start (GTK_BOX (vbox27), table9, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table9), 5);

  start_adj = gtk_adjustment_new (1, 0, 1000000, 1, 10, 10);
  start = gtk_spin_button_new (GTK_ADJUSTMENT (start_adj), 1, 0);
  gtk_widget_ref (start);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "start", start,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (start);
  gtk_table_attach (GTK_TABLE (table9), start, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  end_adj = gtk_adjustment_new (1, 0, 100000, 1, 10, 10);
  end = gtk_spin_button_new (GTK_ADJUSTMENT (end_adj), 1, 0);
  gtk_widget_ref (end);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "end", end,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (end);
  gtk_table_attach (GTK_TABLE (table9), end, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  label44 = gtk_label_new (_("End:"));
  gtk_widget_ref (label44);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "label44", label44,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label44);
  gtk_table_attach (GTK_TABLE (table9), label44, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 5, 5);

  label43 = gtk_label_new (_("Start:"));
  gtk_widget_ref (label43);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "label43", label43,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label43);
  gtk_table_attach (GTK_TABLE (table9), label43, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 5, 5);

  extend = gtk_check_button_new_with_label (_("Extend until the end of the stream"));
  gtk_widget_ref (extend);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "extend", extend,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (extend);
  gtk_box_pack_start (GTK_BOX (vbox27), extend, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (extend), 10);

  hbox33 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox33);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "hbox33", hbox33,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox33);
  gtk_box_pack_start (GTK_BOX (vbox26), hbox33, FALSE, TRUE, 0);

  label46 = gtk_label_new (_("Sense:"));
  gtk_widget_ref (label46);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "label46", label46,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label46);
  gtk_box_pack_start (GTK_BOX (hbox33), label46, FALSE, FALSE, 10);

  sense = gtk_option_menu_new ();
  gtk_widget_ref (sense);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "sense", sense,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sense);
  gtk_box_pack_start (GTK_BOX (hbox33), sense, TRUE, TRUE, 10);
  sense_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Solid to transparent"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (sense_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Transparent to solid"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (sense_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (sense), sense_menu);

  hbox32 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox32);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "hbox32", hbox32,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox32);
  gtk_box_pack_start (GTK_BOX (vbox26), hbox32, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox32), 5);

  frame58 = gtk_frame_new (NULL);
  gtk_widget_ref (frame58);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "frame58", frame58,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame58);
  gtk_box_pack_start (GTK_BOX (hbox32), frame58, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame58), GTK_SHADOW_NONE);

  edit = gtk_button_new_with_label (_("Edit Filtered Stream"));
  gtk_widget_ref (edit);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "edit", edit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (edit);
  gtk_box_pack_start (GTK_BOX (hbox32), edit, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (edit), 5);

  close = gnome_stock_button (GNOME_STOCK_BUTTON_CLOSE);
  gtk_widget_ref (close);
  gtk_object_set_data_full (GTK_OBJECT (sample_filter), "close", close,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (close);
  gtk_box_pack_start (GTK_BOX (hbox32), close, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (close), 5);

  gtk_signal_connect (GTK_OBJECT (sample_filter), "delete_event",
                      GTK_SIGNAL_FUNC (on_sample_filter_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (start), "changed",
                      GTK_SIGNAL_FUNC (on_start_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (end), "changed",
                      GTK_SIGNAL_FUNC (on_end_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (extend), "toggled",
                      GTK_SIGNAL_FUNC (on_extend_toggled),
                      v);
  gtk_signal_connect (GTK_OBJECT (sense), "clicked",
                      GTK_SIGNAL_FUNC (on_sense_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (edit), "clicked",
                      GTK_SIGNAL_FUNC (on_edit_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (close), "clicked",
                      GTK_SIGNAL_FUNC (on_close_clicked),
                      v);

  return sample_filter;
}


void SampleEditor::setValues() {
    SampleFilter *sf=(SampleFilter *)stream;
    GtkEntry *start=GTK_ENTRY(lookup("start"));
    GtkEntry *end=GTK_ENTRY(lookup("end"));
    GtkEntry *sense=GTK_ENTRY(lookup("sense"));

    char buff[1024];

    snprintf(buff,1024,"%d",sf->getStart());
    gtk_entry_set_text(start,buff);
    snprintf(buff,1024,"%d",sf->getEnd());
    gtk_entry_set_text(end,buff);
}

void SampleEditor::show() {
    widget=create_sample_filter(this);
    gtk_widget_show(widget);
    setValues();
}

void SampleEditor::notice(char *event, Subject *source) {

}

void SampleEditor::action(char *action) {
    FilterImStream *fis=(FilterImStream *)stream;
    if (!strcmp(action,"EDIT")) {
        ImStream *s=fis->getFiltered();
	GuiVisitor visitor;
	s->accept(&visitor);
    }else if (!strcmp(action,"SENSE")) {

    }
}

Prototype *SampleEditor::clone() {
    return new SampleEditor();
}

void SampleEditor::setStart(int i) {
    SampleFilter *sf=(SampleFilter *)stream;
    sf->setStart(i);
}

void SampleEditor::setEnd(int i) {
    SampleFilter *sf=(SampleFilter *)stream;
    sf->setEnd(i);
    end=i;
}

void SampleEditor::setExtend(bool x) {
    SampleFilter *sf=(SampleFilter *)stream;
    if (x) {
        sf->setEnd(-1);
    }else{
        sf->setEnd(end);
    }
}

extern "C" {

    class proxy {
    public:
	proxy() {
	    register_stream_editor(new SampleFilter(), new SampleEditor());
	}
    };

    proxy editor;
}
