// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// Sample filter for ganso

/**
 * This sample filter makes an stream more and more transparent, it's usefull
 * as the filter for the top image stream of a concurrent image stream, as
 * it allows to make a transition between top stream and next stream of
 * such a concurrent image stream. The top one makes more and more transparent,
 * disappearing, and the next becames more and more visible as the top makes
 * transparent.
 */

#include "filter.h"

#define NAME "sample filter"
#define VERSION "0.5"
#define COPYRIGHT "Copyright (C) Ruben Lopez Gomez"
#define DESCRIPTION "This is part of the GAnSO sample plugin"

static char *nextName() {
    static int next=0;
    char name[1024];
    snprintf(name,1023,"samplefilter%d",next++);
    return strdup(name);
}

static char *authors[]={
    "Ruben Lopez Gomez",
    NULL
};

SampleFilter::SampleFilter() {
    start=0;
    end=-1; // end of the stream
    pos=0;
    sense=SOLID_TO_TRANSPARENT;
}

static void setTransparency(Image *target, int decay) {
    int w,h,x,y;
    Pixel *targetbuff=target->frameBuffer();
    w=target->width();
    h=target->height();
    for (y=0;y<h;y++)
    {
	Pixel *trow=targetbuff;
	for (x=0;x<w;x++)
	{
	    int alpha=trow->a;
	    alpha-=decay;
	    if (alpha<0) alpha=0;
            trow->a=alpha;
	    trow++;
	}
	targetbuff+=target->width();
    }
}

int SampleFilter::interpolate() {
    int e=end;
    int diff,res;
    if (e==-1) {
        e=filtered->size();
    }
    diff=e-start;
    if (diff<=0) return 0;

    if (pos<start) {
	res=0;
    }else{
	res=(0xFF*((pos-start)+1))/diff;
    }

    if (sense==TRANSPARENT_TO_SOLID) {
        res=0xFF-res;
    }
    return res;
}

void SampleFilter::processNext(Image *im) {
    Image *tmp=new Image(im->width(), im->height());
    filtered->nextImage(tmp);
    setTransparency(tmp,interpolate());
    tmp->copy(im,0,0);
    pos++;
}

void SampleFilter::processRewind() {
    pos=0;
}

void SampleFilter::processSeek(int offset, SeekType type) {
    switch (type) {
    case SSEEK_SET: pos=offset;
    case SSEEK_CUR: pos+=offset;
    }
    if (pos<0) pos=0;
}

char* SampleFilter::getFilterName() {
    return NAME;
}

char* SampleFilter::getFilterVersion() {
    return VERSION;
}

char *SampleFilter::getFilterCopyright() {
    return COPYRIGHT;
}

char **SampleFilter::getFilterAuthors() {
    return authors;
}

char *SampleFilter::getFilterDescription() {
    return DESCRIPTION;
}

Prototype *SampleFilter::clone() {
    char *next=nextName();
    SampleFilter *s=new SampleFilter();
    s->setName(next);
    free(next);
    return s;
}

void SampleFilter::writeSettings(xmlNodePtr xml_node) {
    char _start[40];
    char _end[40];
    char *_sense=NULL;

    snprintf(_start,39,"%d",start);
    snprintf(_end,39,"%d",end);
    if (sense==TRANSPARENT_TO_SOLID) {
        _sense="Transparent to solid";
    }else{
        _sense="Solid to transparent";
    }

    xmlNewChild(xml_node,NULL,(xmlChar*)"start",(xmlChar*)_start);
    xmlNewChild(xml_node,NULL,(xmlChar*)"end",(xmlChar*)_end);
    xmlNewChild(xml_node,NULL,(xmlChar*)"sense",(xmlChar*)_sense);

}

void SampleFilter::readSettings(xmlNodePtr xml_node) {
    xmlNodePtr child=xml_node->childs;
    printf("reading start...\n");
    if (child==NULL) return;
    start=atoi((char*)xmlNodeGetContent(child));
    child=child->next;
    printf("reading end...\n");
    if (child==NULL) return;
    end=atoi((char*)xmlNodeGetContent(child));
    child=child->next;
    printf("reading sense...\n");
    if (child==NULL) return;
    char *_sense=(char*)xmlNodeGetContent(child);
    if (!strcmp(_sense,"Transparent to solid")) {
        sense=TRANSPARENT_TO_SOLID;
    }else if (!strcmp(_sense,"Solid to transparent")) {
	sense=SOLID_TO_TRANSPARENT;
    }
    printf("OK!\n");
}


// Instaling this template automatically when dinamic loaded.
extern "C" {

    class proxy {
    public:
	proxy() {
	    register_video_filter(new SampleFilter());
	}
    };

    proxy filter;
}
