// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "fileimstream.h"
#include <imstreamvisitor.h>

FileImStream::FileImStream(char *name, VideoSource *source):ImStream(name) {
    this->source=source;
    this->first=0;
    this->last=source->size();
    this->repeat=1;
    this->position=0;
    this->loop=0;
}

void FileImStream::nextImage (Image *buffer) {
    if (loop<repeat) {
	Image *im = source->read(position+first);
	im->copy(buffer,0,0);
	if (position<(last-first)) {
	    position++;
	}else{
	    loop++;
	    position=0;
	}
    }else{
	printf("FileImStream ERROR: Attempt to read beyond end of stream [ps=%d,lp=%d]\n",position,loop);
    }
}

void FileImStream::rewind () {
    position=0;
    loop=0;
}

void FileImStream::seek (int offset, SeekType type) {
    switch(type) {
    case SSEEK_SET:
	position=offset%(last-first);
	loop=offset/(last-first);
        break;
    case SSEEK_CUR:
	loop+=(position+offset)/(last-first);
	position=(position+offset)%(last-first);
        break;
    }
}

int FileImStream::size () {return (last-first)*repeat;}

void FileImStream::accept(ImStreamVisitor *visitor) {
    visitor->visitFileImStream(this);
}

void FileImStream::setFirst(int first) {
    this->first=first;
}

void FileImStream::setLast(int last) {
    if (last<=source->size()) {
	this->last=last;
    }else{
        this->last=source->size();
    }
}
void FileImStream::setRepeat(int repeat) {
    this->repeat=repeat;
}
