// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "image.h"
#include "filevisitor.h"

FileVisitor::FileVisitor(xmlNodePtr xml_node) {
    this->node=xml_node;
}

void FileVisitor::visitFileImStream     (FileImStream *s) {
    char buffer[400];

    xmlNodePtr tree=xmlNewChild(node,NULL,(xmlChar*)"file",NULL);
    xmlSetProp(tree,(xmlChar*)"name",(xmlChar*)s->getName());

    xmlNewChild(tree,NULL,(xmlChar*)"filename",(xmlChar*)s->getSource()->getName());

    snprintf(buffer,399,"%d",s->getFirst());
    xmlNewChild(tree,NULL,(xmlChar*)"first",(xmlChar*)buffer);

    snprintf(buffer,399,"%d",s->getLast());
    xmlNewChild(tree,NULL,(xmlChar*)"last",(xmlChar*)buffer);

    snprintf(buffer,399,"%d",s->getRepeat());
    xmlNewChild(tree,NULL,(xmlChar*)"repeat",(xmlChar*)buffer);
}

void FileVisitor::visitSeqImStream       (SeqImStream *s) {
    xmlNodePtr tree=xmlNewChild(node,NULL,(xmlChar*)"seq",NULL);
    xmlSetProp(tree,(xmlChar*)"name",(xmlChar*)s->getName());
    char padding[40];
    // Here must take into account the paddings
    slist<ImStream*> streams=s->getStreams();
    slist<ImStream*>::iterator iter;
    for (iter=streams.begin();iter!=streams.end();iter++) {
	ImStream *stream=*iter;
	snprintf(padding,39,"%d",s->paddingOf(stream));
	xmlNodePtr item=xmlNewChild(tree,NULL,(xmlChar*)"seqitem",NULL);
	xmlSetProp(item,(xmlChar*)"padding",(xmlChar*)padding);
        FileVisitor fv(item);
	stream->accept(&fv);
    }
}

void FileVisitor::visitConcurrentImStream(ConcurrentImStream *s) {
    xmlNodePtr tree=xmlNewChild(node,NULL,(xmlChar*)"conc",NULL);
    xmlSetProp(tree,(xmlChar*)"name",(xmlChar*)s->getName());
    slist<ImStream*> streams=s->getStreams();
    slist<ImStream*>::iterator iter;
    for (iter=streams.begin();iter!=streams.end();iter++) {
	ImStream *stream=*iter;
	// Now the concurrent items has no attributes, but maybe yes in the
	// future, so, to keep compatibility:
	xmlNodePtr item=xmlNewChild(tree,NULL,(xmlChar*)"concitem",NULL);
        FileVisitor fv(item);
	stream->accept(&fv);
    }
}

void FileVisitor::visitFilterImStream   (FilterImStream *s) {
    xmlNodePtr tree=xmlNewChild(node,NULL,(xmlChar*)"filter",NULL);
    xmlSetProp(tree,(xmlChar*)"name",(xmlChar*)s->getName());
    xmlSetProp(tree,(xmlChar*)"filtername",(xmlChar*)s->getFilterName());
    FileVisitor fv(tree);
    s->getFiltered()->accept(&fv);
    // Must create a subtree with the properties and ask the filter to
    // fill it
    xmlNodePtr settings=xmlNewChild(tree,NULL,(xmlChar*)"settings",NULL);
    s->writeSettings(settings);
}
