// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/**
 * FILTERED IMAGE STREAM
 */

#ifndef __FILTERIMSTREAM_H__
#define __FILTERIMSTREAM_H__

#include <prototype.h>
#include "imstream.h"
#include <parser.h> // XML parser

class FilterImStream: public ImStream, public Prototype {
protected:
    ImStream *filtered;

    /* all filters must implement this */
    virtual void   processNext   (Image *im)=0;
    virtual void   processRewind ()=0;
    virtual void   processSeek   (int offset, SeekType type)=0;
public:
    FilterImStream();
    void setFiltered(ImStream *filtered) {this->filtered=filtered;}
    ImStream* getFiltered() { return this->filtered; }
    void nextImage (Image *buffer);
    void rewind    ();
    void seek      (int offset, SeekType type);
    int  size      ();
    void accept(ImStreamVisitor *visitor);

    /* all filters must implement this: */
    virtual char* getFilterName()=0;
    virtual char* getFilterVersion()=0;
    virtual char* getFilterCopyright()=0;
    virtual char**getFilterAuthors()=0;
    virtual Prototype *clone()=0;

    /* Optional */
    virtual char* getFilterDescription() {return "No description";}
    virtual void writeSettings(xmlNodePtr xml_node) {}
    virtual void readSettings(xmlNodePtr xml_node) {}
};

/* This function allows modules to register their filters */
extern void register_video_filter(FilterImStream *filter);

/* This function allows the program to see the registered codecs */
slist<FilterImStream *> get_video_filters();


#endif
