// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#define MAX_XML_COMPRESSION 9

#include <parser.h> // XML parser
#include "gansofile.h"
#include "filevisitor.h"
#include "fileimstream.h"
#include "seqimstream.h"
#include "concurrentimstream.h"
#include "filterimstream.h"
#include "image.h"
#include "videoproxy.h"

void ganso_write_file(ImStream *stream, const char *filename) {
    xmlDocPtr doc;
    xmlNodePtr cur;
    doc=xmlNewDoc((xmlChar*)"1.0");
    xmlSetDocCompressMode(doc,MAX_XML_COMPRESSION);
    doc->root=xmlNewDocNode(doc,NULL,(xmlChar*)"GAnSO-Movie",NULL);
    cur=doc->root;
    FileVisitor fv(cur);
    stream->accept(&fv);

    xmlSaveFile(filename, doc);
}

static ImStream *parseNode(xmlNodePtr node) {
    ImStream *result=NULL;
    if (node == NULL) return NULL;
    if (!strcmp((char*)node->name,"file")) {
	xmlNodePtr fname=node->childs;
	if (fname==NULL) return NULL;
	xmlNodePtr first=fname->next;
	xmlNodePtr last=first->next;
	xmlNodePtr repeat=last->next;
	result=new FileImStream((char*)xmlGetProp(node,(xmlChar*)"name"),
				new VideoProxy((char*)xmlNodeGetContent(fname))
			       );

	if (first!=NULL) ((FileImStream*)result)->setFirst(atoi((char*)xmlNodeGetContent(first)));
        if (last!=NULL) ((FileImStream*)result)->setLast(atoi((char*)xmlNodeGetContent(last)));
        if (repeat!=NULL) ((FileImStream*)result)->setRepeat(atoi((char*)xmlNodeGetContent(repeat)));

    }else if (!strcmp((char*)node->name,"seq")) {
	result=new SeqImStream((char*)xmlGetProp(node,(xmlChar*)"name"));
	xmlNodePtr child=node->childs;
	while(child!=NULL) {
            int padding=atoi((char*)xmlGetProp(child,(xmlChar*)"padding"));
	    xmlNodePtr substream=child->childs;
	    if (substream==NULL) {printf("ERROR:substream=NULL\n");return NULL;}
	    ImStream *ss=parseNode(substream);
	    if (ss!=NULL) {
                ((SeqImStream*)result)->append(ss,padding);
	    }else{
                // warning, file corrupted
		printf("SEQ:warning, file corrupted\n");
	    }
            child=child->next;
	}
    }else if (!strcmp((char*)node->name,"conc")) {
	result=new ConcurrentImStream((char*)xmlGetProp(node,(xmlChar*)"name"));
	xmlNodePtr child=node->childs;
	while(child!=NULL) {
            //Ignoring the child for now...
	    xmlNodePtr substream=child->childs;
	    if (substream==NULL) return NULL;
	    ImStream *ss=parseNode(substream);
	    if (ss!=NULL) {
                ((CompImStream*)result)->append(ss);
	    }else{
		// warning, file corrupted
		printf("CONC: warning, empty concurrent im stream\n");
	    }
            child=child->next;
	}
    }else if (!strcmp((char*)node->name,"filter")) {
	slist<FilterImStream*> filterlist=get_video_filters();
	slist<FilterImStream*>::iterator iter;
	char *thefilter=(char*)xmlGetProp(node,(xmlChar*)"filtername");
	for (iter=filterlist.begin();iter!=filterlist.end();iter++) {
	    FilterImStream* f=*iter;
	    if (!strcmp(f->getFilterName(),thefilter)) {
                FilterImStream *filter=(FilterImStream*)f->clone();
		result=filter;
		filter->setName((char*)xmlGetProp(node,(xmlChar*)"name"));
		xmlNodePtr child=node->childs;
		if (child == NULL) {printf("no subnode\n");return NULL;}
		ImStream *ss=parseNode(child);
		if (ss == NULL) {printf("subnode corrupted\n");return NULL;}
		filter->setFiltered(ss);
		child=child->next;
		if (child==NULL) {printf("no settings");return NULL;}
		if (strcmp((char*)child->name,"settings")) {
		    //warning: file correupted
		    printf("FILTER: warning: file corrupted!\n");
                    return NULL;
		}
                filter->readSettings(child);
	    }
	}

    }else{
        // ERROR: unknown node
	printf("ERROR: unknown node: %s\n",node->name);
    }

    return result;
}

ImStream *ganso_read_file(const char *filename) {
    ImStream *result=NULL;
    xmlDocPtr doc;
    xmlNodePtr cur;

    doc=xmlParseFile(filename);
    if (doc ==NULL) return NULL;

    cur = doc->root;
    if (cur==NULL) {
	// error: document contains no data
        printf("error: document contains no data");
        xmlFreeDoc(doc);
        return NULL;
    }

    if (strcmp((char*)cur->name, "GAnSO-Movie")) {
	// error: document is not a GAnSO composition
        printf("error: document is not a GAnSO composition");
        xmlFreeDoc(doc);
        return NULL;
    }

    result=parseNode(cur->childs);
    xmlFreeDoc(doc);
    return result;
}
