// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "gui_visitor.h"
#include "comp_view.h"
#include "select_template.h"
#include "select_filter.h"

void CompView::updateStreams() {
    CompImStream *domain=(CompImStream *)stream;
    clearVisualList();
    slist<ImStream*> streams=domain->getStreams();
    slist<ImStream*>::iterator i;
    for (i=streams.begin();i!=streams.end();i++) {
        updateOneStream(*i);
    }
}

ImStream *CompView::getSelectedStream() {
    CompImStream *domain=(CompImStream *)stream;
    if (selected==-1) {
	return NULL;
    }else{
	slist<ImStream*> streams=domain->getStreams();
	if (selected>=(int)streams.size()) {
	    // FATAL ERROR
	    return NULL;
	}
	slist<ImStream*>::iterator i=streams.begin();
	for (int j=0;j<selected;j++) i++;
        return *i;
    }
}

void CompView::setSelected(int selected) {
    this->selected=selected;
}

void CompView::editCurrent() {
    ImStream *s=getSelectedStream();
    if (s) {
        GuiVisitor visitor;
	s->accept(&visitor);
    }else{
	//ERROR: NONE SELECTED
    }
}

void CompView::filterCurrent() {
    ImStream *s=getSelectedStream();
    if (s) {
	SelectFilter *sf=SelectFilter::getInstance();
	sf->addObserver(this);
        sf->show();
    }else{
	//ERROR: NONE SELECTED
    }
}

void CompView::buildNewStream() {
    SelectTemplate *builder=SelectTemplate::getInstance();
    builder->addObserver(this);
    builder->show();
}

void CompView::notice(char *event, Subject *source) {
    CompImStream *domain=(CompImStream *)stream;
    if (!strcmp(event,COMP_APPEND)
	|| !strcmp(event,COMP_REMOVE)
	|| !strcmp(event,COMP_UPDATE)) {
	updateStreams();
    }else if (!strcmp(event,SELECT_TEMPLATE_BUILD_COMPLETE)) {
	SelectTemplate *builder=SelectTemplate::getInstance();
	domain->append(builder->getStream());
	builder->removeObserver(this);
    }else if (!strcmp(event,SELECT_FILTER_DONE)) {
	SelectFilter *sf=SelectFilter::getInstance();
	ImStream *selStream=getSelectedStream();
	FilterImStream *fis=sf->getFilter();
        fis->setFiltered(selStream);
	domain->update(selStream,fis);
    }else
        noticeMore(event,source);
}
