// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include <seqimstream.h>
#include <concurrentimstream.h>
#include <image.h>
#include <videoproxy.h>

#include "select_file.h"
#include "error_dialog.h"

#include "default_template.h"
#include "pixmaps/conc_imBig.xpm"
#include "pixmaps/fileBig.xpm"
#include "pixmaps/seq_imBig.xpm"
#include <stdio.h>
#include "support.h"

static char *authors[]={
    "Ruben Lopez <ryu@gpul.org>",
    NULL
};

static char *concNext() {
    static int cur=0;
    char s[1024];
    snprintf(s,1023,"conc%03d",cur++);
    return strdup(s);
}

static char *fileNext() {
    static int cur=0;
    char s[1024];
    snprintf(s,1023,"file%03d",cur++);
    return strdup(s);
}

static char *seqNext() {
    static int cur=0;
    char s[1024];
    snprintf(s,1023,"seq%03d",cur++);
    return strdup(s);
}

DefaultTemplate::DefaultTemplate(DefaultTemplateStream stream) {
    type=stream;
    this->stream=NULL;
}

char ** DefaultTemplate::getIcon() {
    switch (type) {
    case DT_FILE:
        return fileBig_xpm;
        break;
    case DT_SEQUENTIAL:
        return seq_imBig_xpm;
        break;
    case DT_CONCURRENT:
        return conc_imBig_xpm;
        break;
    }
    return NULL;
}

// Mandatory Information
char *DefaultTemplate::getName() {
    switch (type) {
    case DT_FILE:
        return _("File");
        break;
    case DT_SEQUENTIAL:
        return _("Sequential");
        break;
    case DT_CONCURRENT:
        return _("Concurrent");
        break;
    }
    return NULL;
}

char *DefaultTemplate::getVersion() {
    return "1.0";
}

char *DefaultTemplate::getCopyright() {
    return "Copyright (C) Ruben Lopez";
}

char **DefaultTemplate::getAuthors() {
    return authors;
}

void DefaultTemplate::build() {
    switch (this->type) {
    case DT_FILE:
	{
	    SelectFile *selector = new SelectFile(_("Select an image"));
	    selector->addObserver(this);
	}
        break;
    case DT_SEQUENTIAL:
    case DT_CONCURRENT:
        notify(TEMPLATE_BUILD_COMPLETE);
        break;
    }

}

// Stream getting
ImStream *DefaultTemplate::getStream() {
    switch (type) {
    case DT_FILE:
        return stream;
        break;
    case DT_SEQUENTIAL:
        return new SeqImStream(seqNext());
        break;
    case DT_CONCURRENT:
        return new ConcurrentImStream(concNext());
        break;
    }
    return NULL;
}

void DefaultTemplate::notice(char *event, Subject *source) {
    if (strstr(event,"FILE_SELECTED")==event) {
	char s[1024];
	sscanf(event,"FILE_SELECTED: %s",s);

	VideoProxy *p = new VideoProxy(s);
	if (p->hasLoader()) {
	    stream = new FileImStream(fileNext(),p);
	    notify(TEMPLATE_BUILD_COMPLETE);
	}else{
            new ErrorDialog("Unsupported format");
	}
    }
}
