// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "support.h"
#include "stdarg.h"
#include "error_dialog.h"
#include "pixmaps/gnome-error.xpm"

static gboolean
on_error_dialog_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        ErrorDialog     *v)
{
    delete v;
    return TRUE;
}


static void
on_error_ok_clicked                    (GtkButton       *button,
                                        ErrorDialog     *v)
{
    delete v;
}


GtkWidget*
create_error_dialog (ErrorDialog *v)
{
  GtkWidget *error_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox34;
  GtkWidget *pixmap1;
  GtkWidget *error_msg;
  GtkWidget *dialog_action_area1;
  GtkWidget *error_ok;

  error_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (error_dialog), "error_dialog", error_dialog);
  gtk_window_set_title (GTK_WINDOW (error_dialog), _("Error"));
  gtk_window_set_policy (GTK_WINDOW (error_dialog), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (error_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (error_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  hbox34 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox34);
  gtk_object_set_data_full (GTK_OBJECT (error_dialog), "hbox34", hbox34,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox34);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox34, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox34), 10);

  pixmap1 = gnome_pixmap_new_from_xpm_d (gnome_error_xpm);
  gtk_widget_ref (pixmap1);
  gtk_object_set_data_full (GTK_OBJECT (error_dialog), "pixmap1", pixmap1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (hbox34), pixmap1, FALSE, TRUE, 0);

  error_msg = gtk_label_new (_("Error"));
  gtk_widget_ref (error_msg);
  gtk_object_set_data_full (GTK_OBJECT (error_dialog), "error_msg", error_msg,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (error_msg);
  gtk_box_pack_start (GTK_BOX (hbox34), error_msg, FALSE, FALSE, 10);

  dialog_action_area1 = GTK_DIALOG (error_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (error_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  error_ok = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_ref (error_ok);
  gtk_object_set_data_full (GTK_OBJECT (error_dialog), "error_ok", error_ok,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (error_ok);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), error_ok, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (error_dialog), "delete_event",
                      GTK_SIGNAL_FUNC (on_error_dialog_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (error_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_error_ok_clicked),
                      v);

  return error_dialog;
}

ErrorDialog::ErrorDialog(char *fmt, ...) {
    char buffer[4096];
    va_list ap;
                        
    va_start(ap,fmt);
    vsnprintf(buffer, 4095, fmt, ap);
    va_end(ap);

    widget=create_error_dialog(this);
    gtk_label_set_text(GTK_LABEL(lookup("error_msg")), buffer);
    gtk_widget_show(widget);
    gtk_grab_add(widget);
}

void ErrorDialog::action(char *action) {

}
