// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "gui_visitor.h"
#include "filterimstream.h"
#include "filter_view.h"

static gboolean
on_filter_editor_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        FilterView      *fv)
{
    delete fv;
    return TRUE;
}


static void
on_edit_clicked                        (GtkButton       *button,
					FilterView      *fv)
{
    fv->action("EDIT");
}


static void
on_close_clicked                       (GtkButton       *button,
                                        FilterView      *fv)
{
    delete fv;
}

static GtkWidget*
create_filter_editor (FilterView *v)
{
  GtkWidget *filter_editor;
  GtkWidget *vbox25;
  GtkWidget *hbox29;
  GtkWidget *label39;
  GtkWidget *name;
  GtkWidget *frame54;
  GtkWidget *frames_target;
  GtkWidget *hbox30;
  GtkWidget *frame56;
  GtkWidget *edit;
  GtkWidget *close;

  filter_editor = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (filter_editor), "filter_editor", filter_editor);
  gtk_window_set_title (GTK_WINDOW (filter_editor), _("Default filter editor"));

  vbox25 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox25);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "vbox25", vbox25,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox25);
  gtk_container_add (GTK_CONTAINER (filter_editor), vbox25);
  gtk_container_set_border_width (GTK_CONTAINER (vbox25), 10);

  hbox29 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox29);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "hbox29", hbox29,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox29);
  gtk_box_pack_start (GTK_BOX (vbox25), hbox29, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox29), 10);

  label39 = gtk_label_new (_("Name:"));
  gtk_widget_ref (label39);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "label39", label39,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label39);
  gtk_box_pack_start (GTK_BOX (hbox29), label39, FALSE, FALSE, 4);

  name = gtk_label_new (_("*name*"));
  gtk_widget_ref (name);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "name", name,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (name);
  gtk_box_pack_start (GTK_BOX (hbox29), name, TRUE, TRUE, 10);
  gtk_label_set_justify (GTK_LABEL (name), GTK_JUSTIFY_LEFT);

  frame54 = gtk_frame_new (_("Frames"));
  gtk_widget_ref (frame54);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "frame54", frame54,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame54);
  gtk_box_pack_start (GTK_BOX (vbox25), frame54, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame54), 5);

  frames_target = gtk_frame_new (NULL);
  gtk_widget_ref (frames_target);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "frames_target", frames_target,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frames_target);
  gtk_container_add (GTK_CONTAINER (frame54), frames_target);
  gtk_container_set_border_width (GTK_CONTAINER (frames_target), 5);
  gtk_frame_set_shadow_type (GTK_FRAME (frames_target), GTK_SHADOW_NONE);

  hbox30 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox30);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "hbox30", hbox30,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox30);
  gtk_box_pack_start (GTK_BOX (vbox25), hbox30, FALSE, TRUE, 0);

  frame56 = gtk_frame_new (NULL);
  gtk_widget_ref (frame56);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "frame56", frame56,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame56);
  gtk_box_pack_start (GTK_BOX (hbox30), frame56, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame56), GTK_SHADOW_NONE);

  edit = gtk_button_new_with_label (_("Edit Filtered Stream"));
  gtk_widget_ref (edit);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "edit", edit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (edit);
  gtk_box_pack_start (GTK_BOX (hbox30), edit, FALSE, FALSE, 3);

  close = gnome_stock_button (GNOME_STOCK_BUTTON_CLOSE);
  gtk_widget_ref (close);
  gtk_object_set_data_full (GTK_OBJECT (filter_editor), "close", close,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (close);
  gtk_box_pack_start (GTK_BOX (hbox30), close, FALSE, FALSE, 3);

  gtk_signal_connect (GTK_OBJECT (filter_editor), "delete_event",
                      GTK_SIGNAL_FUNC (on_filter_editor_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (edit), "clicked",
                      GTK_SIGNAL_FUNC (on_edit_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (close), "clicked",
                      GTK_SIGNAL_FUNC (on_close_clicked),
                      v);

  return filter_editor;
}

void FilterView::show() {
    widget=create_filter_editor(this);
    gtk_widget_show(widget);
}

void FilterView::action(char *action) {
    FilterImStream *fis=(FilterImStream *)stream;
    if (!strcmp(action,"EDIT")) {
        ImStream *s=fis->getFiltered();
	GuiVisitor visitor;
	s->accept(&visitor);
    }
}

void FilterView::notice(char *event, Subject *source) {

}

Prototype *FilterView::clone() {
    return new FilterView();
}

