// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "gnome_tree.h"
#include "tree_visitor.h"
#include "gui_visitor.h"
#include "preview.h"

static guint detect_user_click(GtkWidget     *widget,
			      GdkEventButton *event,
			      GnomeTree      *tree)
{
    if (event->type == GDK_2BUTTON_PRESS)
	tree->showStream();

    tree->showPreview();
    return FALSE;
}

void GnomeTree::showPreview() {
    Preview::instance()->setStream(stream);
}

GnomeTree::GnomeTree(ImStream *s, char **pixmap, GtkTree *parent) {
    stream=s;
    treeitem=gtk_tree_item_new();
    gtk_widget_show(treeitem);
    setPixmap(pixmap);
    gtk_signal_connect(GTK_OBJECT(treeitem),"button_press_event",
		       GTK_SIGNAL_FUNC (detect_user_click), this);
    this->parent=parent;
    gtk_tree_append(parent,treeitem);

    tree=NULL;
    s->addObserver(this);
}

GnomeTree::~GnomeTree() {
    stream->removeObserver(this);
    TreeList::iterator iter;
    for (iter=children.begin();iter!=children.end();iter++) {
	GnomeTree *child=*iter;
        delete child;
    }
    GList *list=g_list_append(NULL,treeitem);
    gtk_tree_remove_items(parent,list);
    g_list_free(list);
}

GtkTree *GnomeTree::getSubTree() {
    if (tree==NULL) {
	tree=gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(treeitem),tree);
    }
    return GTK_TREE(tree);
}

void GnomeTree::notice(char *event, Subject *source) {
    TreeVisitor visitor(this);
    stream->accept(&visitor);
}

void GnomeTree::showStream() {
    stream->accept(new GuiVisitor());
}

static GtkWidget *buildItemWidget(GtkWidget *pixmap, char *text) {
    GtkWidget *widget=gtk_hbox_new(FALSE,0);
    GtkWidget *label=gtk_label_new(text);
    gtk_widget_show(widget);
    gtk_widget_show(label);
    gtk_widget_show(pixmap);
    gtk_box_pack_start(GTK_BOX(widget),pixmap,FALSE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(widget),label,FALSE,TRUE,5);
    return widget;
}

void GnomeTree::setPixmap(char **data) {
    GdkPixmap *xpm;
    GdkBitmap *xbm;
    xpm=gdk_pixmap_create_from_xpm_d(treeitem->window,&xbm,NULL,data);
    GtkWidget *pixmap=gtk_pixmap_new(xpm,xbm);
    gtk_container_add(GTK_CONTAINER(treeitem),buildItemWidget(pixmap,stream->getName()));
}
