// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "error_dialog.h"
#include <sys/stat.h> // To check de existence of files
#include <gnome.h>
#include <seqimstream.h>
#include <fileimstream.h>
#include <image.h>
#include <videoproxy.h>

#include "pattern_template.h"
#include "pixmaps/pattern.xpm"
#include <stdio.h>

static void
on_wizzard_cancel                      (GnomeDruid      *gnomedruid,
                                        PatternTemplate *pt)
{
    pt->hide();
}


static void
on_directory_changed                   (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    pt->setDirectory(gtk_entry_get_text(GTK_ENTRY(editable)));
}


static void
on_prefix_changed                      (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    pt->setPrefix(gtk_entry_get_text(GTK_ENTRY(editable)));
}


static void
on_suffix_changed                      (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    pt->setSuffix(gtk_entry_get_text(GTK_ENTRY(editable)));
}


static void
on_range_low_changed                   (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    char *s=gtk_entry_get_text(GTK_ENTRY(editable));
    char *end;
    int i=strtol(s,&end,10);
    if (end!=s) {
        pt->setFirst(i);
    }
}


static void
on_range_high_changed                  (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    char *s=gtk_entry_get_text(GTK_ENTRY(editable));
    char *end;
    int i=strtol(s,&end,10);
    if (end!=s) {
        pt->setLast(i);
    }
}


static void
on_fill_zeros_toggled                  (GtkToggleButton *togglebutton,
                                        PatternTemplate *pt)
{
    pt->setFill(togglebutton->active == 1);
}


static void
on_check_existence_toggled             (GtkToggleButton *togglebutton,
                                        PatternTemplate *pt)
{

}


static void
on_ordering_clicked                    (GtkButton       *button,
                                        PatternTemplate *pt)
{

}


static gboolean
on_pattern_template_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        PatternTemplate *pt)
{
    pt->hide();
    return TRUE;
}

static void
on_finish_page_finish                  (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        PatternTemplate *pt)
{
    pt->action("FINISH");
}


static void
on_directory_page_prepare              (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        PatternTemplate *pt)
{
    pt->action("PREPARE_DIRECTORY");
}


static void
on_pattern_page_prepare                (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        PatternTemplate *pt)
{
    pt->action("PREPARE_PATTERN");
}


static void
on_sort_page_prepare                   (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        PatternTemplate *pt)
{
    pt->action("PREPARE_SORT");
}

static void
on_digits_changed                      (GtkEditable     *editable,
                                        PatternTemplate *pt)
{
    char *s=gtk_entry_get_text(GTK_ENTRY(editable));
    char *end;
    int i=strtol(s,&end,10);
    if (end!=s && i<100) {
        pt->setDigits(i);
    }
}

static GtkWidget*
create_pattern_template (PatternTemplate *v)
{
  GtkWidget *pattern_template;
  GtkWidget *wizzard;
  GtkWidget *start_page;
  GdkColor start_page_bg_color = { 0, 6425, 6425, 28784 };
  GdkColor start_page_textbox_color = { 0, 65535, 65535, 65535 };
  GdkColor start_page_logo_bg_color = { 0, 65535, 65535, 65535 };
  GdkColor start_page_title_color = { 0, 65535, 65535, 65535 };
  GtkWidget *directory_page;
  GdkColor directory_page_bg_color = { 0, 6425, 6425, 28784 };
  GdkColor directory_page_logo_bg_color = { 0, 65535, 65535, 65535 };
  GdkColor directory_page_title_color = { 0, 65535, 65535, 65535 };
  GtkWidget *druid_vbox1;
  GtkWidget *vbox22;
  GtkWidget *alignment1;
  GtkWidget *accellabel1;
  GtkWidget *hbox25;
  GtkWidget *label30;
  GtkWidget *fileentry1;
  GtkWidget *directory;
  GtkWidget *pattern_page;
  GdkColor pattern_page_bg_color = { 0, 6425, 6425, 28784 };
  GdkColor pattern_page_logo_bg_color = { 0, 65535, 65535, 65535 };
  GdkColor pattern_page_title_color = { 0, 65535, 65535, 65535 };
  GtkWidget *druid_vbox2;
  GtkWidget *vbox23;
  GtkWidget *alignment2;
  GtkWidget *label31;
  GtkWidget *frame53;
  GtkWidget *table8;
  GtkWidget *label32;
  GtkWidget *label33;
  GtkWidget *prefix;
  GtkWidget *suffix;
  GtkWidget *label34;
  GtkWidget *hbox26;
  GtkObject *range_low_adj;
  GtkWidget *range_low;
  GtkWidget *label35;
  GtkObject *range_high_adj;
  GtkWidget *range_high;
  GtkWidget *alignment4;
  GtkWidget *hbox28;
  GtkWidget *label38;
  GtkObject *digits_adj;
  GtkWidget *digits;
  GtkWidget *fill_zeros;
  GtkWidget *alignment3;
  GtkWidget *check_existence;
  GtkWidget *sort_page;
  GdkColor sort_page_bg_color = { 0, 6425, 6425, 28784 };
  GdkColor sort_page_logo_bg_color = { 0, 65535, 65535, 65535 };
  GdkColor sort_page_title_color = { 0, 65535, 65535, 65535 };
  GtkWidget *druid_vbox3;
  GtkWidget *vbox24;
  GtkWidget *alignment5;
  GtkWidget *label36;
  GtkWidget *hbox27;
  GtkWidget *label37;
  GtkWidget *ordering;
  GtkWidget *ordering_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *finish_page;
  GdkColor finish_page_bg_color = { 0, 6425, 6425, 28784 };
  GdkColor finish_page_textbox_color = { 0, 65535, 65535, 65535 };
  GdkColor finish_page_logo_bg_color = { 0, 65535, 65535, 65535 };
  GdkColor finish_page_title_color = { 0, 65535, 65535, 65535 };

  pattern_template = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (pattern_template), "pattern_template", pattern_template);
  gtk_window_set_title (GTK_WINDOW (pattern_template), _("Pattern Template"));

  wizzard = gnome_druid_new ();
  gtk_widget_ref (wizzard);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "wizzard", wizzard,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (wizzard);
  gtk_container_add (GTK_CONTAINER (pattern_template), wizzard);

  start_page = gnome_druid_page_start_new ();
  gtk_widget_ref (start_page);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "start_page", start_page,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (start_page);
  gnome_druid_append_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (start_page));
  gnome_druid_set_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (start_page));
  gnome_druid_page_start_set_bg_color (GNOME_DRUID_PAGE_START (start_page), &start_page_bg_color);
  gnome_druid_page_start_set_textbox_color (GNOME_DRUID_PAGE_START (start_page), &start_page_textbox_color);
  gnome_druid_page_start_set_logo_bg_color (GNOME_DRUID_PAGE_START (start_page), &start_page_logo_bg_color);
  gnome_druid_page_start_set_title_color (GNOME_DRUID_PAGE_START (start_page), &start_page_title_color);
  gnome_druid_page_start_set_title (GNOME_DRUID_PAGE_START (start_page), _("Simple pattern based Stream builder"));
  gnome_druid_page_start_set_text (GNOME_DRUID_PAGE_START (start_page), _("This template will allow you to create a simple sequential\n image stream composed by a group of image files defined\nby a pattern.\nPress \"next\" to continue."));

  directory_page = gnome_druid_page_standard_new_with_vals ("", NULL);
  gtk_widget_ref (directory_page);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "directory_page", directory_page,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show_all (directory_page);
  gnome_druid_append_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (directory_page));
  gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (directory_page), &directory_page_bg_color);
  gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (directory_page), &directory_page_logo_bg_color);
  gnome_druid_page_standard_set_title_color (GNOME_DRUID_PAGE_STANDARD (directory_page), &directory_page_title_color);
  gnome_druid_page_standard_set_title (GNOME_DRUID_PAGE_STANDARD (directory_page), _("Directory where the images are placed"));

  druid_vbox1 = GNOME_DRUID_PAGE_STANDARD (directory_page)->vbox;
  gtk_widget_ref (druid_vbox1);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "druid_vbox1", druid_vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druid_vbox1);

  vbox22 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox22);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "vbox22", vbox22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox22);
  gtk_box_pack_start (GTK_BOX (druid_vbox1), vbox22, TRUE, TRUE, 0);

  alignment1 = gtk_alignment_new (0, 0, 1, 1);
  gtk_widget_ref (alignment1);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "alignment1", alignment1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment1);
  gtk_box_pack_start (GTK_BOX (vbox22), alignment1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (alignment1), 5);

  accellabel1 = gtk_accel_label_new (_("Select here the directory where your images are placed..."));
  gtk_widget_ref (accellabel1);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "accellabel1", accellabel1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (accellabel1);
  gtk_container_add (GTK_CONTAINER (alignment1), accellabel1);

  hbox25 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox25);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "hbox25", hbox25,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox25);
  gtk_box_pack_start (GTK_BOX (vbox22), hbox25, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox25), 25);

  label30 = gtk_label_new (_("Directory:"));
  gtk_widget_ref (label30);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label30", label30,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label30);
  gtk_box_pack_start (GTK_BOX (hbox25), label30, FALSE, FALSE, 0);

  fileentry1 = gnome_file_entry_new (NULL, NULL);
  gtk_widget_ref (fileentry1);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "fileentry1", fileentry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry1);
  gtk_box_pack_start (GTK_BOX (hbox25), fileentry1, TRUE, TRUE, 10);

  directory = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry1));
  gtk_widget_ref (directory);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "directory", directory,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (directory);

  pattern_page = gnome_druid_page_standard_new_with_vals ("", NULL);
  gtk_widget_ref (pattern_page);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "pattern_page", pattern_page,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show_all (pattern_page);
  gnome_druid_append_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (pattern_page));
  gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (pattern_page), &pattern_page_bg_color);
  gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (pattern_page), &pattern_page_logo_bg_color);
  gnome_druid_page_standard_set_title_color (GNOME_DRUID_PAGE_STANDARD (pattern_page), &pattern_page_title_color);
  gnome_druid_page_standard_set_title (GNOME_DRUID_PAGE_STANDARD (pattern_page), _("Pattern options"));

  druid_vbox2 = GNOME_DRUID_PAGE_STANDARD (pattern_page)->vbox;
  gtk_widget_ref (druid_vbox2);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "druid_vbox2", druid_vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druid_vbox2);

  vbox23 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox23);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "vbox23", vbox23,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox23);
  gtk_box_pack_start (GTK_BOX (druid_vbox2), vbox23, TRUE, TRUE, 0);

  alignment2 = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_ref (alignment2);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "alignment2", alignment2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment2);
  gtk_box_pack_start (GTK_BOX (vbox23), alignment2, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (alignment2), 5);

  label31 = gtk_label_new (_("Select here the way the pattern will find images."));
  gtk_widget_ref (label31);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label31", label31,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label31);
  gtk_container_add (GTK_CONTAINER (alignment2), label31);

  frame53 = gtk_frame_new (NULL);
  gtk_widget_ref (frame53);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "frame53", frame53,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame53);
  gtk_box_pack_start (GTK_BOX (vbox23), frame53, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame53), 20);

  table8 = gtk_table_new (4, 2, FALSE);
  gtk_widget_ref (table8);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "table8", table8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table8);
  gtk_container_add (GTK_CONTAINER (frame53), table8);
  gtk_container_set_border_width (GTK_CONTAINER (table8), 10);

  label32 = gtk_label_new (_("Text prefix:"));
  gtk_widget_ref (label32);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label32", label32,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label32);
  gtk_table_attach (GTK_TABLE (table8), label32, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label33 = gtk_label_new (_("Text suffix:"));
  gtk_widget_ref (label33);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label33", label33,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label33);
  gtk_table_attach (GTK_TABLE (table8), label33, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  prefix = gtk_entry_new ();
  gtk_widget_ref (prefix);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "prefix", prefix,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prefix);
  gtk_table_attach (GTK_TABLE (table8), prefix, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 10, 5);
  gtk_entry_set_text (GTK_ENTRY (prefix), _("image"));

  suffix = gtk_entry_new ();
  gtk_widget_ref (suffix);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "suffix", suffix,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (suffix);
  gtk_table_attach (GTK_TABLE (table8), suffix, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 10, 5);
  gtk_entry_set_text (GTK_ENTRY (suffix), _(".ppm"));

  label34 = gtk_label_new (_("Range:"));
  gtk_widget_ref (label34);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label34", label34,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label34);
  gtk_table_attach (GTK_TABLE (table8), label34, 0, 1, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  hbox26 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox26);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "hbox26", hbox26,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox26);
  gtk_table_attach (GTK_TABLE (table8), hbox26, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 10, 5);

  range_low_adj = gtk_adjustment_new (0, 0, 1e+09, 1, 10, 10);
  range_low = gtk_spin_button_new (GTK_ADJUSTMENT (range_low_adj), 1, 0);
  gtk_widget_ref (range_low);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "range_low", range_low,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (range_low);
  gtk_box_pack_start (GTK_BOX (hbox26), range_low, TRUE, TRUE, 0);

  label35 = gtk_label_new (_("..."));
  gtk_widget_ref (label35);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label35", label35,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label35);
  gtk_box_pack_start (GTK_BOX (hbox26), label35, FALSE, FALSE, 5);

  range_high_adj = gtk_adjustment_new (100, 0, 1e+09, 1, 10, 10);
  range_high = gtk_spin_button_new (GTK_ADJUSTMENT (range_high_adj), 1, 0);
  gtk_widget_ref (range_high);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "range_high", range_high,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (range_high);
  gtk_box_pack_start (GTK_BOX (hbox26), range_high, TRUE, TRUE, 0);

  alignment4 = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_ref (alignment4);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "alignment4", alignment4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment4);
  gtk_table_attach (GTK_TABLE (table8), alignment4, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 10, 5);

  hbox28 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox28);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "hbox28", hbox28,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox28);
  gtk_container_add (GTK_CONTAINER (alignment4), hbox28);

  label38 = gtk_label_new (_("Digits:"));
  gtk_widget_ref (label38);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label38", label38,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label38);
  gtk_box_pack_start (GTK_BOX (hbox28), label38, FALSE, FALSE, 5);

  digits_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  digits = gtk_spin_button_new (GTK_ADJUSTMENT (digits_adj), 1, 0);
  gtk_widget_ref (digits);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "digits", digits,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (digits);
  gtk_box_pack_start (GTK_BOX (hbox28), digits, TRUE, TRUE, 5);

  fill_zeros = gtk_check_button_new_with_label (_("Fill with zeros"));
  gtk_widget_ref (fill_zeros);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "fill_zeros", fill_zeros,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fill_zeros);
  gtk_table_attach (GTK_TABLE (table8), fill_zeros, 0, 1, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fill_zeros), TRUE);

  alignment3 = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_ref (alignment3);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "alignment3", alignment3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment3);
  gtk_box_pack_start (GTK_BOX (vbox23), alignment3, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (alignment3), 20);

  check_existence = gtk_check_button_new_with_label (_("Check the existence of these images before continuing"));
  gtk_widget_ref (check_existence);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "check_existence", check_existence,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (check_existence);
  gtk_container_add (GTK_CONTAINER (alignment3), check_existence);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_existence), TRUE);

  sort_page = gnome_druid_page_standard_new_with_vals ("", NULL);
  gtk_widget_ref (sort_page);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "sort_page", sort_page,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show_all (sort_page);
  gnome_druid_append_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (sort_page));
  gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (sort_page), &sort_page_bg_color);
  gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (sort_page), &sort_page_logo_bg_color);
  gnome_druid_page_standard_set_title_color (GNOME_DRUID_PAGE_STANDARD (sort_page), &sort_page_title_color);
  gnome_druid_page_standard_set_title (GNOME_DRUID_PAGE_STANDARD (sort_page), _("Order of images"));

  druid_vbox3 = GNOME_DRUID_PAGE_STANDARD (sort_page)->vbox;
  gtk_widget_ref (druid_vbox3);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "druid_vbox3", druid_vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druid_vbox3);

  vbox24 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox24);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "vbox24", vbox24,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox24);
  gtk_box_pack_start (GTK_BOX (druid_vbox3), vbox24, TRUE, TRUE, 0);

  alignment5 = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_ref (alignment5);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "alignment5", alignment5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (alignment5);
  gtk_box_pack_start (GTK_BOX (vbox24), alignment5, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (alignment5), 10);

  label36 = gtk_label_new (_("Select here the order of the images"));
  gtk_widget_ref (label36);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label36", label36,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label36);
  gtk_container_add (GTK_CONTAINER (alignment5), label36);

  hbox27 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox27);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "hbox27", hbox27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox27);
  gtk_box_pack_start (GTK_BOX (vbox24), hbox27, FALSE, FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (hbox27), 20);

  label37 = gtk_label_new (_("Ordering:"));
  gtk_widget_ref (label37);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "label37", label37,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label37);
  gtk_box_pack_start (GTK_BOX (hbox27), label37, FALSE, FALSE, 5);
  gtk_misc_set_alignment (GTK_MISC (label37), 0, 0.5);

  ordering = gtk_option_menu_new ();
  gtk_widget_ref (ordering);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "ordering", ordering,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ordering);
  gtk_box_pack_start (GTK_BOX (hbox27), ordering, TRUE, TRUE, 10);
  ordering_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Ascending"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (ordering_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Descending"));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (ordering_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (ordering), ordering_menu);

  finish_page = gnome_druid_page_finish_new ();
  gtk_widget_ref (finish_page);
  gtk_object_set_data_full (GTK_OBJECT (pattern_template), "finish_page", finish_page,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (finish_page);
  gnome_druid_append_page (GNOME_DRUID (wizzard), GNOME_DRUID_PAGE (finish_page));
  gnome_druid_page_finish_set_bg_color (GNOME_DRUID_PAGE_FINISH (finish_page), &finish_page_bg_color);
  gnome_druid_page_finish_set_textbox_color (GNOME_DRUID_PAGE_FINISH (finish_page), &finish_page_textbox_color);
  gnome_druid_page_finish_set_logo_bg_color (GNOME_DRUID_PAGE_FINISH (finish_page), &finish_page_logo_bg_color);
  gnome_druid_page_finish_set_title_color (GNOME_DRUID_PAGE_FINISH (finish_page), &finish_page_title_color);
  gnome_druid_page_finish_set_title (GNOME_DRUID_PAGE_FINISH (finish_page), _("Simple pattern based Stream builder"));
  gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (finish_page), _("The stream was successfully composed, click\n\"finish\" to add it to the composition."));

  gtk_signal_connect (GTK_OBJECT (pattern_template), "delete_event",
                      GTK_SIGNAL_FUNC (on_pattern_template_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (wizzard), "cancel",
                      GTK_SIGNAL_FUNC (on_wizzard_cancel),
                      v);
  gtk_signal_connect (GTK_OBJECT (directory_page), "prepare",
                      GTK_SIGNAL_FUNC (on_directory_page_prepare),
                      v);
  gtk_signal_connect (GTK_OBJECT (directory), "changed",
                      GTK_SIGNAL_FUNC (on_directory_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (pattern_page), "prepare",
                      GTK_SIGNAL_FUNC (on_pattern_page_prepare),
                      v);
  gtk_signal_connect (GTK_OBJECT (prefix), "changed",
                      GTK_SIGNAL_FUNC (on_prefix_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (suffix), "changed",
                      GTK_SIGNAL_FUNC (on_suffix_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (range_low), "changed",
                      GTK_SIGNAL_FUNC (on_range_low_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (range_high), "changed",
                      GTK_SIGNAL_FUNC (on_range_high_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (digits), "changed",
                      GTK_SIGNAL_FUNC (on_digits_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (fill_zeros), "toggled",
                      GTK_SIGNAL_FUNC (on_fill_zeros_toggled),
                      v);
  gtk_signal_connect (GTK_OBJECT (check_existence), "toggled",
                      GTK_SIGNAL_FUNC (on_check_existence_toggled),
                      v);
  gtk_signal_connect (GTK_OBJECT (sort_page), "prepare",
                      GTK_SIGNAL_FUNC (on_sort_page_prepare),
                      v);
  gtk_signal_connect (GTK_OBJECT (ordering), "clicked",
                      GTK_SIGNAL_FUNC (on_ordering_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (finish_page), "finish",
                      GTK_SIGNAL_FUNC (on_finish_page_finish),
                      v);

  return pattern_template;
}


PatternTemplate::PatternTemplate() {
    directory=NULL;
    prefix=NULL;
    suffix=NULL;
    first=0;last=100;
    digits=1;
    fill_zeros=true;
    type=PT_ASCENDING;
    hidden=true;
    widget=NULL;
    stream=NULL;
}


static char *authors[]={
    "Ruben Lopez <ryu@gpul.org>",
    NULL
};

#define PATVERSION "0.1"
#define COPYRIGHT "Copyright (C) Ruben Lopez"
#define NAME "Pattern Template"

char ** PatternTemplate::getIcon() {
    return pattern_xpm;
}

// Mandatory Information
char *PatternTemplate::getName() {
    return NAME;
}

char *PatternTemplate::getVersion() {
    return PATVERSION;
}

char *PatternTemplate::getCopyright() {
    return COPYRIGHT;
}

char **PatternTemplate::getAuthors() {
    return authors;
}

void PatternTemplate::build() {
    if (hidden) {
	widget=create_pattern_template(this);
	gtk_widget_show(widget);
        gtk_grab_add(widget);
        hidden=false;
    }
}

// Stream getting
ImStream *PatternTemplate::getStream() {
    return stream;
}

void PatternTemplate::checkDirectory() {
    if (directory==NULL) {
        gnome_druid_set_buttons_sensitive(GNOME_DRUID(lookup("wizzard")), TRUE, FALSE, TRUE);
    }else{
        gnome_druid_set_buttons_sensitive(GNOME_DRUID(lookup("wizzard")), TRUE, TRUE, TRUE);
    }
}

void PatternTemplate::checkPattern() {
    if (prefix==NULL || suffix==NULL) {
        gnome_druid_set_buttons_sensitive(GNOME_DRUID(lookup("wizzard")), TRUE, FALSE, TRUE);
    }else{
        gnome_druid_set_buttons_sensitive(GNOME_DRUID(lookup("wizzard")), TRUE, TRUE, TRUE);
    }

}

void PatternTemplate::action(char *action) {
    struct stat status;
    bool was_error=false;
    if (!strcmp(action,"FINISH")) {
        stream=new SeqImStream("patternseq");
	for (int i=first;i<=last;i++) {
	    char s[1024];
	    if (fill_zeros){
		char num[40], num2[512];
		snprintf(num,39,"%d",i);
		int has=strlen(num);
		for (int j=0;j<digits-has;j++) {
                    num2[j]='0';
		}
		if (has>digits) has=digits;
                snprintf(num2+digits-has,has+1,"%d",i);
		snprintf(s,1023,"%s%s%s",prefix,num2,suffix);
	    }else{
		snprintf(s,1023,"%s%d%s",prefix,i,suffix);
	    }
	    char path[1024];
	    if (directory[strlen(directory)-1]=='/') {
		snprintf(path,1023,"%s%s",directory,s);
	    }else{
		snprintf(path,1023,"%s/%s",directory,s);
	    }
	    if (stat(path,&status)==0) {
		if (status.st_mode & S_IFREG) {
		    FileImStream *cs=new FileImStream(s,new VideoProxy(path));
		    ((SeqImStream*)stream)->append(cs,0);
		}else{
		    //error, not a regular file
                    was_error=true;
		}
	    }else{
		//error, file doesn't exist
                was_error=true;
	    }
	}
	if (was_error) {
	    new ErrorDialog("Some images couldn't be processed\n");
	}
	notify(TEMPLATE_BUILD_COMPLETE);
        hide();
    }else if (!strcmp(action,"PREPARE_DIRECTORY")) {
        checkDirectory();
    }else if (!strcmp(action,"PREPARE_PATTERN")) {
        setPrefix(gtk_entry_get_text(GTK_ENTRY(lookup("prefix"))));
        setSuffix(gtk_entry_get_text(GTK_ENTRY(lookup("suffix"))));
        checkPattern();
    }else if (!strcmp(action,"PREPARE_SORT")) {

    }
}

void PatternTemplate::hide() {
    if (!hidden) {
        gtk_grab_remove(widget);
	gtk_widget_destroy(widget);
        hidden=true;
    }
}

void PatternTemplate::setDirectory(char *dir) {
    if (directory) {
	free(directory);
        directory=NULL;
    }
    if (dir[0]!=0)
	directory=strdup(dir);

    checkDirectory();
}

void PatternTemplate::setPrefix(char *prefix) {
    if (this->prefix) {
	free(this->prefix);
        this->prefix=NULL;
    }
    if (prefix[0]!=0) {
        this->prefix=strdup(prefix);
    }

    checkPattern();
}

void PatternTemplate::setSuffix(char *suffix) {
    if (this->suffix) {
	free(this->suffix);
        this->suffix=NULL;
    }
    if (suffix[0]!=0) {
        this->suffix=strdup(suffix);
    }

    checkPattern();
}

void PatternTemplate::setFirst(int first) {
    this->first=first;
}

void PatternTemplate::setLast(int last) {
    this->last=last;
}

void PatternTemplate::setFill(bool fill) {
    fill_zeros=fill;
}

void PatternTemplate::setDigits(int digits) {
    this->digits=digits;
}

void PatternTemplate::setSorting(PatternTemplateSorting type) {
    this->type=type;
}
