// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "preview.h"
#include "gnome_driver.h"
#include "pixmaps/play.xpm"
#include "pixmaps/pause.xpm"
#include "pixmaps/stop.xpm"

gboolean
on_preview_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        Preview         *preview)
{
    delete(preview);
    return TRUE;
}


void
on_play_clicked                        (GtkButton       *button,
                                        Preview         *preview)
{
    preview->action("PLAY");
}


void
on_pause_clicked                       (GtkButton       *button,
                                        Preview         *preview)
{
    preview->action("PAUSE");
}


void
on_stop_clicked                        (GtkButton       *button,
                                        Preview         *preview)
{
    preview->action("STOP");
}


void
on_position_changed                    (GtkEditable     *editable,
                                        Preview         *preview)
{
    preview->action("POSITION");
}

static gboolean
on_videoarea_expose             (GtkWidget       *widget,
				 GdkEventExpose  *event,
				 Preview         *preview)
{
    preview->action("EXPOSE");
    return FALSE;
}

GtkWidget*
create_preview (Preview *v)
{
  GtkWidget *preview;
  GtkWidget *vbox1;
  GtkWidget *hbox4;
  GtkWidget *frame1;
  GtkWidget *videoarea;
  GtkWidget *hbox1;
  GtkWidget *play;
  GtkWidget *playxpm;
  GtkWidget *pause;
  GtkWidget *pausexpm;
  GtkWidget *stop;
  GtkWidget *stopxpm;
  GtkObject *position_adj;
  GtkWidget *position;
  GtkWidget *hbox3;
  GtkWidget *label1;
  GtkWidget *size;

  preview = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (preview), "preview", preview);
  gtk_container_set_border_width (GTK_CONTAINER (preview), 10);
  gtk_window_set_title (GTK_WINDOW (preview), _("Preview"));

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (preview), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (preview), vbox1);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (preview), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox4, TRUE, FALSE, 0);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (preview), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (hbox4), frame1, TRUE, FALSE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_IN);

  videoarea = gtk_drawing_area_new ();
  gtk_widget_ref (videoarea);
  gtk_object_set_data_full (GTK_OBJECT (preview), "videoarea", videoarea,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (videoarea);
  gtk_container_add (GTK_CONTAINER (frame1), videoarea);
  gtk_widget_set_usize (videoarea, 64, 64);
  gtk_widget_set_events (videoarea, GDK_EXPOSURE_MASK);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (preview), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 10);

  play = gtk_button_new ();
  gtk_widget_ref (play);
  gtk_object_set_data_full (GTK_OBJECT (preview), "play", play,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (play);
  gtk_box_pack_start (GTK_BOX (hbox1), play, FALSE, FALSE, 3);
  gtk_button_set_relief (GTK_BUTTON (play), GTK_RELIEF_NONE);

  playxpm = gnome_pixmap_new_from_xpm_d (play_xpm);
  gtk_widget_ref (playxpm);
  gtk_object_set_data_full (GTK_OBJECT (preview), "playxpm", playxpm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (playxpm);
  gtk_container_add (GTK_CONTAINER (play), playxpm);

  pause = gtk_button_new ();
  gtk_widget_ref (pause);
  gtk_object_set_data_full (GTK_OBJECT (preview), "pause", pause,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pause);
  gtk_box_pack_start (GTK_BOX (hbox1), pause, FALSE, FALSE, 3);
  gtk_button_set_relief (GTK_BUTTON (pause), GTK_RELIEF_NONE);

  pausexpm = gnome_pixmap_new_from_xpm_d (pause_xpm);
  gtk_widget_ref (pausexpm);
  gtk_object_set_data_full (GTK_OBJECT (preview), "pausexpm", pausexpm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pausexpm);
  gtk_container_add (GTK_CONTAINER (pause), pausexpm);

  stop = gtk_button_new ();
  gtk_widget_ref (stop);
  gtk_object_set_data_full (GTK_OBJECT (preview), "stop", stop,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (stop);
  gtk_box_pack_start (GTK_BOX (hbox1), stop, FALSE, FALSE, 3);
  gtk_button_set_relief (GTK_BUTTON (stop), GTK_RELIEF_NONE);

  stopxpm = gnome_pixmap_new_from_xpm_d (stop_xpm);
  gtk_widget_ref (stopxpm);
  gtk_object_set_data_full (GTK_OBJECT (preview), "stopxpm", stopxpm,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (stopxpm);
  gtk_container_add (GTK_CONTAINER (stop), stopxpm);

  position_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  position = gtk_spin_button_new (GTK_ADJUSTMENT (position_adj), 1, 0);
  gtk_widget_ref (position);
  gtk_object_set_data_full (GTK_OBJECT (preview), "position", position,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (position);
  gtk_box_pack_start (GTK_BOX (hbox1), position, FALSE, FALSE, 3);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (preview), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (hbox1), hbox3, TRUE, TRUE, 0);

  label1 = gtk_label_new ("/");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (preview), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox3), label1, FALSE, FALSE, 2);

  size = gtk_label_new ("-");
  gtk_widget_ref (size);
  gtk_object_set_data_full (GTK_OBJECT (preview), "size", size,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (size);
  gtk_box_pack_start (GTK_BOX (hbox3), size, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (preview), "delete_event",
                      GTK_SIGNAL_FUNC (on_preview_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (play), "clicked",
                      GTK_SIGNAL_FUNC (on_play_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (pause), "clicked",
                      GTK_SIGNAL_FUNC (on_pause_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (stop), "clicked",
                      GTK_SIGNAL_FUNC (on_stop_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (position), "changed",
                      GTK_SIGNAL_FUNC (on_position_changed),
                      v);
  gtk_signal_connect (GTK_OBJECT (videoarea), "expose_event",
                      GTK_SIGNAL_FUNC (on_videoarea_expose),
                      v);

  return preview;
}

Preview *Preview::pr_instance=NULL;

Preview::Preview() {
    this->widget=create_preview(this);
    gtk_widget_show(this->widget);
    gc=gdk_gc_new(this->widget->window);
    stream=NULL;
    position=0;
    pixmap=NULL;
    timer=-1;
    image=NULL;
}

gint time_handler(Preview *v) {
    v->showNext();
    return TRUE;
}

void Preview::action(char *action) {
    if (stream) {
	if (!strcmp(action,"PLAY")) {
            timer=gtk_timeout_add(50,(GtkFunction)time_handler,this);
	}else if (!strcmp(action,"PAUSE")) {
	    if (timer>0) {
		gtk_timeout_remove(timer);
                timer=-1;
	    }
	}else if (!strcmp(action,"STOP")) {
            this->action("PAUSE");
	    position=0;
	    stream->rewind();
	    showNext();
	}else if (!strcmp(action,"POSITION")) {
	    char *txt=gtk_entry_get_text(GTK_ENTRY(lookup("position")));
	    char *end=NULL;
	    int value=(int)strtol(txt,&end,10);
	    setPosition(value);
	}else if (!strcmp(action,"EXPOSE")) {
            repaint();
	}
    }
}

void Preview::setStream(ImStream *stream) {
    if (this->stream) {
	this->stream->removeObserver(this);
    }
    this->stream=stream;
    this->stream->addObserver(this);
    stream->rewind();
    position=0;
    recalculate();
}

void Preview::recalculate() {
    if (stream) {
	char str[1024];
	gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(lookup("position")))->upper=stream->size()-1;
	snprintf(str,1024,"%d",stream->size()-1);
	gtk_label_set_text(GTK_LABEL(lookup("size")),str);
	setPosition(position);
    }
}

void Preview::setPosition(int position) {
    if (stream) {
	this->position=position;
	stream->seek(position,SSEEK_SET);
	showNext();
    }
}

void Preview::showNext() {
    if (stream && image) {
	position++;
	stream->nextImage(image);
	draw_image_on_drawable(image, pixmap, gc, gtk_widget_get_colormap(widget));
	repaint();
	if (position>=stream->size() && timer>0) {
            this->action("STOP");
	}
    }
}

void Preview::setSize(int width,int height) {
    if (width<10) width=10;
    if (height<10) height=10;
    this->width=width;
    this->height=height;
    if (image) delete (image);
    image=new Image(width,height);
    gtk_widget_set_usize(lookup("videoarea"),width,height);
    pixmap=gdk_pixmap_new(widget->window, width, height, gdk_visual_get_best_depth());
    setPosition(position);
}

void Preview::repaint() {
    if (pixmap && stream) {
	GdkDrawable *p=lookup("videoarea")->window;
	gdk_draw_pixmap(p,gc,pixmap, 0,0,0,0,width,height);
    }
}

Preview *Preview::instance() {
    if (pr_instance==NULL) {
        pr_instance=new Preview();
    }
    return pr_instance;
}

Preview::~Preview() {
    if (stream) {
        stream->removeObserver(this);
    }
}

void Preview::notice(char *event, Subject *source) {
    recalculate();
}
