// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "select_codec.h"
#include "progress_metter.h"

static void
on_clist1_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        SelectCodec     *sc)
{
    sc->setSelected(row);
}


static void
on_config_codec_clicked                (GtkButton       *button,
                                        SelectCodec     *sc)
{
    sc->action("CONFIG_CODEC");
}


static void
on_about_codec_clicked                 (GtkButton       *button,
                                        SelectCodec     *sc)
{
    sc->action("ABOUT_CODEC");
}

static gboolean
on_select_codec_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        SelectCodec     *sc)
{

    delete (sc);
    return TRUE;
}


static void
on_codec_ok_clicked                    (GtkButton       *button,
                                        SelectCodec     *sc)
{
    sc->action("OK");
}


static void
on_codec_cancel_clicked                (GtkButton       *button,
                                        SelectCodec     *sc)
{
    delete (sc);
}


static GtkWidget*
create_select_codec (SelectCodec *v)
{
  GtkWidget *select_codec;
  GtkWidget *frame31;
  GtkWidget *vbox15;
  GtkWidget *frame32;
  GtkWidget *vbox16;
  GtkWidget *scrolledwindow7;
  GtkWidget *clist1;
  GtkWidget *label15;
  GtkWidget *hbox13;
  GtkWidget *config_codec;
  GtkWidget *about_codec;
  GtkWidget *frame34;
  GtkWidget *hbox12;
  GtkWidget *codec_ok;
  GtkWidget *codec_cancel;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  select_codec = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (select_codec), "select_codec", select_codec);
  gtk_widget_set_usize (select_codec, 300, 220);
  gtk_window_set_title (GTK_WINDOW (select_codec), _("Select a codec"));

  frame31 = gtk_frame_new (NULL);
  gtk_widget_ref (frame31);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "frame31", frame31,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame31);
  gtk_container_add (GTK_CONTAINER (select_codec), frame31);
  gtk_container_set_border_width (GTK_CONTAINER (frame31), 5);
  gtk_frame_set_shadow_type (GTK_FRAME (frame31), GTK_SHADOW_NONE);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox15);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "vbox15", vbox15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox15);
  gtk_container_add (GTK_CONTAINER (frame31), vbox15);

  frame32 = gtk_frame_new (_("Codecs"));
  gtk_widget_ref (frame32);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "frame32", frame32,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame32);
  gtk_box_pack_start (GTK_BOX (vbox15), frame32, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame32), 3);

  vbox16 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox16);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "vbox16", vbox16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox16);
  gtk_container_add (GTK_CONTAINER (frame32), vbox16);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow7);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "scrolledwindow7", scrolledwindow7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow7);
  gtk_box_pack_start (GTK_BOX (vbox16), scrolledwindow7, TRUE, TRUE, 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  clist1 = gtk_clist_new (1);
  gtk_widget_ref (clist1);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "clist1", clist1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (clist1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), clist1);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist1));

  label15 = gtk_label_new (_("Codecs"));
  gtk_widget_ref (label15);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "label15", label15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label15);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label15);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox13);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "hbox13", hbox13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox16), hbox13, FALSE, TRUE, 3);

  config_codec = gtk_button_new_with_label (_("Setup"));
  gtk_widget_ref (config_codec);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "config_codec", config_codec,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (config_codec);
  gtk_box_pack_start (GTK_BOX (hbox13), config_codec, FALSE, FALSE, 3);
  gtk_tooltips_set_tip (tooltips, config_codec, _("Let's you configure this codec to fit your needs"), NULL);

  about_codec = gtk_button_new_with_label (_("About"));
  gtk_widget_ref (about_codec);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "about_codec", about_codec,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (about_codec);
  gtk_box_pack_start (GTK_BOX (hbox13), about_codec, FALSE, FALSE, 3);
  gtk_tooltips_set_tip (tooltips, about_codec, _("Shows more information about this plugin"), NULL);

  frame34 = gtk_frame_new (NULL);
  gtk_widget_ref (frame34);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "frame34", frame34,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame34);
  gtk_box_pack_start (GTK_BOX (vbox15), frame34, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame34), 3);
  gtk_frame_set_shadow_type (GTK_FRAME (frame34), GTK_SHADOW_NONE);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox12);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "hbox12", hbox12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox12);
  gtk_container_add (GTK_CONTAINER (frame34), hbox12);

  codec_ok = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_ref (codec_ok);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "codec_ok", codec_ok,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (codec_ok);
  gtk_box_pack_start (GTK_BOX (hbox12), codec_ok, FALSE, FALSE, 3);
  GTK_WIDGET_SET_FLAGS (codec_ok, GTK_CAN_DEFAULT);

  codec_cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_ref (codec_cancel);
  gtk_object_set_data_full (GTK_OBJECT (select_codec), "codec_cancel", codec_cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (codec_cancel);
  gtk_box_pack_start (GTK_BOX (hbox12), codec_cancel, FALSE, FALSE, 3);
  GTK_WIDGET_SET_FLAGS (codec_cancel, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (select_codec), "delete_event",
                      GTK_SIGNAL_FUNC (on_select_codec_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (clist1), "select_row",
                      GTK_SIGNAL_FUNC (on_clist1_select_row),
                      v);
  gtk_signal_connect (GTK_OBJECT (config_codec), "clicked",
                      GTK_SIGNAL_FUNC (on_config_codec_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (about_codec), "clicked",
                      GTK_SIGNAL_FUNC (on_about_codec_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (codec_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_codec_ok_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (codec_cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_codec_cancel_clicked),
                      v);

  gtk_widget_grab_focus (codec_ok);
  gtk_widget_grab_default (codec_ok);
  gtk_object_set_data (GTK_OBJECT (select_codec), "tooltips", tooltips);

  return select_codec;
}


SelectCodec::SelectCodec(Creation *creation) {
    domain=creation;
    this->widget = create_select_codec(this);
    gtk_widget_show(widget);
    GtkCList *clist=GTK_CLIST(lookup("clist1"));
    slist<VideoCodec*> codecs = get_video_codecs();
    slist<VideoCodec*>::iterator iter;
    for (iter=codecs.begin(); iter!=codecs.end(); iter++) {
	VideoCodec *vc=*iter;
	char *name=vc->getName();
        gtk_clist_append(clist,&name);
    }
    selected=NULL;
}

void SelectCodec::action(char *action) {
    if (!strcmp(action,"CONFIG_CODEC")) {
	if (selected) {
	    selected->configure();
	}
    }else if (!strcmp(action,"ABOUT_CODEC")) {
	if (selected) {
	    GtkWidget *about=gnome_about_new(selected->getName(),
					     selected->getVersion(),
					     selected->getCopyright(),
					     (const char**)selected->getAuthors(),
					     selected->getDescription(),
					     NULL);
            gtk_widget_show(about);
	}
    }else if (!strcmp(action,"OK")) {
	if (selected != NULL) {
	    domain->compose(selected);
            new ProgressMetter(selected);
	    delete(this);
	}else {
	    // Error
	}

    }
}

void SelectCodec::setSelected(int codec) {
    int i=0;
    slist<VideoCodec*> codecs = get_video_codecs();
    slist<VideoCodec*>::iterator iter;
    for (iter=codecs.begin(); iter!=codecs.end(); iter++) {
	if (i==codec) {
	    selected=*iter;
	}
	i++;
    }
}
