// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include "select_filter.h"

static gboolean
on_select_filter_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        SelectFilter    *filter)
{
    filter->hide();
    return TRUE;
}


static void
on_about_filter_clicked                (GtkButton       *button,
                                        SelectFilter    *filter)
{
    filter->action("ABOUT");
}


static void
on_filter_ok_clicked                   (GtkButton       *button,
                                        SelectFilter    *filter)
{
    filter->action("OK");
}


static void
on_filter_Cancel_clicked               (GtkButton       *button,
                                        SelectFilter    *filter)
{
    filter->hide();
}


static void
on_filter_list_select_row              (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        SelectFilter    *filter)
{
    filter->setSelected(row);
}


static GtkWidget*
create_select_filter (SelectFilter *v)
{
  GtkWidget *select_filter;
  GtkWidget *frame45;
  GtkWidget *vbox18;
  GtkWidget *frame46;
  GtkWidget *vbox19;
  GtkWidget *scrolledwindow9;
  GtkWidget *filter_list;
  GtkWidget *label29;
  GtkWidget *hbox21;
  GtkWidget *about_filter;
  GtkWidget *frame47;
  GtkWidget *hbox22;
  GtkWidget *filter_ok;
  GtkWidget *filter_Cancel;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  select_filter = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (select_filter), "select_filter", select_filter);
  gtk_window_set_title (GTK_WINDOW (select_filter), _("Select a filter"));

  frame45 = gtk_frame_new (NULL);
  gtk_widget_ref (frame45);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "frame45", frame45,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame45);
  gtk_container_add (GTK_CONTAINER (select_filter), frame45);
  gtk_container_set_border_width (GTK_CONTAINER (frame45), 5);
  gtk_frame_set_shadow_type (GTK_FRAME (frame45), GTK_SHADOW_NONE);

  vbox18 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox18);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "vbox18", vbox18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox18);
  gtk_container_add (GTK_CONTAINER (frame45), vbox18);

  frame46 = gtk_frame_new (_("Filters"));
  gtk_widget_ref (frame46);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "frame46", frame46,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame46);
  gtk_box_pack_start (GTK_BOX (vbox18), frame46, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame46), 3);

  vbox19 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox19);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "vbox19", vbox19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox19);
  gtk_container_add (GTK_CONTAINER (frame46), vbox19);

  scrolledwindow9 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow9);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "scrolledwindow9", scrolledwindow9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow9);
  gtk_box_pack_start (GTK_BOX (vbox19), scrolledwindow9, TRUE, TRUE, 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow9), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  filter_list = gtk_clist_new (1);
  gtk_widget_ref (filter_list);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "filter_list", filter_list,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (filter_list);
  gtk_container_add (GTK_CONTAINER (scrolledwindow9), filter_list);
  gtk_clist_set_column_width (GTK_CLIST (filter_list), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (filter_list));

  label29 = gtk_label_new (_("Filters"));
  gtk_widget_ref (label29);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "label29", label29,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label29);
  gtk_clist_set_column_widget (GTK_CLIST (filter_list), 0, label29);

  hbox21 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox21);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "hbox21", hbox21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox21, FALSE, TRUE, 3);

  about_filter = gtk_button_new_with_label (_("About"));
  gtk_widget_ref (about_filter);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "about_filter", about_filter,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (about_filter);
  gtk_box_pack_start (GTK_BOX (hbox21), about_filter, FALSE, FALSE, 3);
  gtk_tooltips_set_tip (tooltips, about_filter, _("Shows more information about this plugin"), NULL);

  frame47 = gtk_frame_new (NULL);
  gtk_widget_ref (frame47);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "frame47", frame47,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame47);
  gtk_box_pack_start (GTK_BOX (vbox18), frame47, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame47), 3);
  gtk_frame_set_shadow_type (GTK_FRAME (frame47), GTK_SHADOW_NONE);

  hbox22 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox22);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "hbox22", hbox22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox22);
  gtk_container_add (GTK_CONTAINER (frame47), hbox22);

  filter_ok = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_ref (filter_ok);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "filter_ok", filter_ok,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (filter_ok);
  gtk_box_pack_start (GTK_BOX (hbox22), filter_ok, FALSE, FALSE, 3);
  GTK_WIDGET_SET_FLAGS (filter_ok, GTK_CAN_DEFAULT);

  filter_Cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_ref (filter_Cancel);
  gtk_object_set_data_full (GTK_OBJECT (select_filter), "filter_Cancel", filter_Cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (filter_Cancel);
  gtk_box_pack_start (GTK_BOX (hbox22), filter_Cancel, FALSE, FALSE, 3);
  GTK_WIDGET_SET_FLAGS (filter_Cancel, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (select_filter), "delete_event",
                      GTK_SIGNAL_FUNC (on_select_filter_delete_event),
                      v);
  gtk_signal_connect (GTK_OBJECT (filter_list), "select_row",
                      GTK_SIGNAL_FUNC (on_filter_list_select_row),
                      v);
  gtk_signal_connect (GTK_OBJECT (about_filter), "clicked",
                      GTK_SIGNAL_FUNC (on_about_filter_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (filter_ok), "clicked",
                      GTK_SIGNAL_FUNC (on_filter_ok_clicked),
                      v);
  gtk_signal_connect (GTK_OBJECT (filter_Cancel), "clicked",
                      GTK_SIGNAL_FUNC (on_filter_Cancel_clicked),
                      v);

  gtk_widget_grab_focus (filter_ok);
  gtk_widget_grab_default (filter_ok);
  gtk_object_set_data (GTK_OBJECT (select_filter), "tooltips", tooltips);

  return select_filter;
}


static SelectFilter *instance=NULL;

SelectFilter::SelectFilter() {
    this->widget = create_select_filter(this);
    GtkCList *clist=GTK_CLIST(lookup("filter_list"));
    slist<FilterImStream*> filters = get_video_filters();
    slist<FilterImStream*>::iterator iter;
    for (iter=filters.begin(); iter!=filters.end(); iter++) {
	FilterImStream *fis=*iter;
	char *name=fis->getFilterName();
        gtk_clist_append(clist,&name);
    }
    selected=NULL;
    shown=false;
}

void SelectFilter::action(char *action) {
    if (!strcmp(action,"ABOUT")) {
	if (selected) {
	    GtkWidget *about=gnome_about_new(selected->getFilterName(),
					     selected->getFilterVersion(),
					     selected->getFilterCopyright(),
					     (const char**)selected->getFilterAuthors(),
					     selected->getFilterDescription(),
					     NULL);
            gtk_widget_show(about);
	}
    }else if (!strcmp(action,"OK")) {
	gtk_grab_remove(widget);
	notify(SELECT_FILTER_DONE);
        hide();
    }
}

void SelectFilter::setSelected(int filter) {
    int i=0;
    slist<FilterImStream*> filters = get_video_filters();
    slist<FilterImStream*>::iterator iter;
    for (iter=filters.begin(); iter!=filters.end(); iter++) {
	if (i==filter) {
	    selected=*iter;
	}
	i++;
    }
}

FilterImStream *SelectFilter::getFilter() {
    return (FilterImStream*)selected->clone();
}

SelectFilter *SelectFilter::getInstance() {
    if (instance==NULL) {
        instance=new SelectFilter();
    }
    return instance;
}

void SelectFilter::show() {
    if (!shown) {
	gtk_widget_show(widget);
	gtk_grab_add(widget);
        shown=true;
    }
}

void SelectFilter::hide() {
    if (shown) {
	gtk_grab_remove(widget);
	gtk_widget_hide(widget);
        clear(); // Clear observers list
        shown=false;
    }
}
