// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>
#include <tree_visitor.h>
#include "pixmaps/conc_im.xpm"
#include "pixmaps/file.xpm"
#include "pixmaps/filter.xpm"
#include "pixmaps/seq_im.xpm"

TreeVisitor::TreeVisitor(GnomeTree *tree) {
    this->tree=tree;
    this->gtktree=NULL;
}

TreeVisitor::TreeVisitor(GtkTree *tree) {
    this->tree=NULL;
    this->gtktree=tree;
}

void TreeVisitor::visitFileImStream(FileImStream *s) {
    if (gtktree) {
        new GnomeTree(s,file_xpm,gtktree);
    }
}

bool TreeVisitor::findTree(TreeList *list, ImStream *stream) {
    TreeList::iterator iter;
    for (iter=list->begin();iter!=list->end();iter++) {
	GnomeTree *t=*iter;

        // t can be null if it's a simple image
        if (t!=NULL && t->getStream()==stream) return true;
    }
    return false;
}

bool TreeVisitor::findStream(StreamList *list, ImStream *stream) {
    StreamList::iterator iter;
    for (iter=list->begin();iter!=list->end();iter++) {
        ImStream *s=*iter;
        if (s==stream) return true;
    }
    return false;
}

void TreeVisitor::compare(StreamList &domain, StreamList &newS, TreeList &oldS) {
    TreeList *known=tree->getChildren();
    StreamList::iterator iter;
    TreeList::iterator tIter;
    for (iter=domain.begin();iter!=domain.end();iter++) {
	ImStream *current=*iter;
	if (!findTree(known,current)) {
            newS.insert(newS.end(),current);
	}
    }
    for (tIter=known->begin();tIter!=known->end();tIter++) {
	GnomeTree *current=*tIter;
	if (current!=NULL && !findStream(&domain,current->getStream())) {
	    oldS.insert(oldS.end(),current);
	}
    }
}

void TreeVisitor::visitComposite(CompImStream *domain) {
    StreamList newS;
    TreeList oldS;
    StreamList streams=domain->getStreams();
    compare(streams,newS,oldS);
    StreamList::iterator i;
    TreeList *known=tree->getChildren();
    for (i=newS.begin();i!=newS.end();i++) {
	ImStream *s=*i;
	TreeVisitor tv(tree->getSubTree());
	s->accept(&tv);
        known->insert(known->end(),tv.getTree());
    }
    TreeList::iterator ti;
    for (ti=oldS.begin();ti!=oldS.end();ti++) {
        GnomeTree *gt=*ti;
	known->remove(gt);
        delete gt;
    }
}


void TreeVisitor::visitSeqImStream(SeqImStream *s) {
    if (gtktree) {
        tree=new GnomeTree(s,seq_im_xpm,gtktree);
    }
    visitComposite(s);
}

void TreeVisitor::visitConcurrentImStream(ConcurrentImStream *s) {
    if (gtktree) {
        tree=new GnomeTree(s,conc_im_xpm,gtktree);
    }
    visitComposite(s);
}

void TreeVisitor::visitFilterImStream(FilterImStream *s) {
    if (gtktree) {
        tree=new GnomeTree(s,filter_xpm,gtktree);
    }
    TreeVisitor tv(tree->getSubTree());
    s->getFiltered()->accept(&tv);
}
