// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <main_window.h>

#include <gnome_driver.h>

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>


#include <videoloader.h>
#include "gui/select_codec.h"
#include "gui/default_template.h"
#include "gui/pattern_template.h"
#include "gui/gnome_waiter.h"
#include "gui/preview.h"

#include "config.h"

void link_local() {
    void *h;

    h=dlopen("../codecs/output/video/ucbmpeg/.libs/ucbmpeg.so",RTLD_NOW);
    if (!h) {printf("ERROR: %s\n",dlerror());exit(1);}

    h=dlopen("../plugins/sample/.libs/sample.so",RTLD_NOW);
    if (!h) {printf("ERROR: %s\n",dlerror());exit(1);}

    h=dlopen("../codecs/input/video/png/.libs/png.so",RTLD_NOW);
    if (!h) {printf("ERROR: %s\n",dlerror());exit(1);}

    h=dlopen("../codecs/input/video/ppm/.libs/ppm.so",RTLD_NOW);
    if (!h) {printf("ERROR: %s\n",dlerror());exit(1);}

    h=dlopen("../codecs/input/video/mpeg/.libs/mpeg.so",RTLD_NOW);
    if (!h) {printf("ERROR: %s\n",dlerror());exit(1);}
}

FILE *open_some_config_file() {
    char home_file[1024];
    strncpy(home_file,getenv("HOME"),1023);
    strncat(home_file,"/.ganso/ganso.conf",1023);
    FILE *f=fopen(home_file,"ro");
    if (f==NULL) {
	f=fopen(GANSO_CONFIG,"ro");
	if (f==NULL) {
	    g_warning("No configuration file were found neither at \"%s\" nor at \"%s\"",GANSO_CONFIG,home_file);
	    return NULL;
	}else{
	    return f;
	}
    }else{
	return f;
    }
}

void read_config(char **plug_dir, char **codec_dir, char **dll_ext) {
    FILE *config=open_some_config_file();
    if (config == NULL) {
	return;
    }
    char buffer[1024];
    char var[1024];
    char val[1024];
    while (!feof(config)) {
	fgets(buffer,1023,config);
	sscanf(buffer,"%s = %s",var,val);
	if (!strcmp(var,"PLUGINS_DIR")) *plug_dir=strdup(val);
	if (!strcmp(var,"CODECS_DIR")) *codec_dir=strdup(val);
	if (!strcmp(var,"DLL_EXTENSION")) *dll_ext=strdup(val);
    }
    fclose(config);
}

void read_plugs(char *dir, char *ext) {
    DIR *direct=NULL;
    struct dirent *element=NULL;
    direct = opendir (dir);

    if (direct==NULL) {
        return;
    }

    while ((element = readdir (direct)) != NULL)
    {
	char *file=element->d_name;
	char *pos_extension=strrchr(file,'.');
	if (pos_extension!=NULL) {
	    pos_extension++;
	    if (!strcmp(pos_extension,ext)) {
		char *s=(char*)malloc(strlen(dir)+strlen(file)+2);
		strcpy(s,dir);
		strcat(s,"/");
                strcat(s,file);
		void *h=dlopen(s,RTLD_NOW);
		if (!h) {g_warning("ERROR LOADING SHARED OBJECT [%s]: %s\n",s,dlerror());}
	    }
	}
    }
    closedir (direct);
}

#define MAX_DIR 2048
void link_installed() {
    char *plug_dir=NULL;
    char *codec_dir=NULL;
    char *dll_ext=NULL;
    char codec_fulldir[MAX_DIR];

    read_config(&plug_dir, &codec_dir, &dll_ext);
    if (dll_ext==NULL) {
        dll_ext="so";
    }
    if (plug_dir!=NULL)  read_plugs(plug_dir,dll_ext);
    if (codec_dir!=NULL) {
        snprintf(codec_fulldir,MAX_DIR,"%s/output",codec_dir);
	read_plugs(codec_fulldir,dll_ext);
        snprintf(codec_fulldir,MAX_DIR,"%s/input/video",codec_dir);
	read_plugs(codec_fulldir,dll_ext);
    }
}

int main(int argc, char *argv[])
{
    GnomeDriver::instance()->init(argc,argv);

#ifdef DEVELOPMENT
    printf("DEVELOPMENT MODE\n");
    link_local(); // Loads dinamic loading libraries (aka plugins)
#else
    link_installed(); // Loads dinamic loading libraries (aka plugins)
#endif

    Creation *creation=new Creation();
    new MainWindow(creation);
    register_video_template(new DefaultTemplate(DT_FILE));
    register_video_template(new DefaultTemplate(DT_SEQUENTIAL));
    register_video_template(new DefaultTemplate(DT_CONCURRENT));
    register_video_template(new PatternTemplate());

    InputWaiter::setInstance(new GnomeWaiter());
    Preview::instance();
    GnomeDriver::instance()->main();
    return 0;    
}
