// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include "videocache.h"
#include "image.h"

static VideoCache * _instance=NULL;

VideoCache::VideoCache() {
    cache_size=10;
}

void VideoCache::add(VideoSource *source, int position, Image *im) {
    VideoCacheKey *k=new VideoCacheKey(source,position);
    if (!search(k)) {
	if ((int)images.size() == cache_size) {
	    VideoCacheEntry *entry=*(images.begin());
	    delete entry->first;
	    delete entry->second;
	    delete entry;
	    images.pop_front();
	}
	images.insert(images.end(), new VideoCacheEntry(k,im));
    }
}

Image *VideoCache::search(VideoCacheKey* key) {
    VideoCacheContents::iterator i;
    for (i=images.begin();i!=images.end();i++) {
	VideoCacheEntry *entry=*i;
        VideoCacheKey *k=entry->first;
	if (k->first == key->first && k->second == key->second) {
	    return entry->second;
	}
    }
    return NULL;
}


Image* VideoCache::get(VideoSource *source, int position) {
    VideoCacheKey key(source,position);
    Image *im=search(&key);
    if (im == NULL) {
	im=source->read(position);
	add(source,position,im);
    }
    return im;
}

void VideoCache::clear() {
    while ( images.size() > 0 ) {
	images.pop_front();
    }
}

VideoCache *VideoCache::instance() {
    if (!_instance) {
	_instance=new VideoCache();
    }
    return _instance;
}

void VideoCache::setCacheSize(int size) {
    // Free overflow images
    while (cache_size>size) {
	images.pop_front();
	cache_size--;
    }
    cache_size=size;
}
