// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <slist.h>
#include <videocodec.h>
#include <view.h>
#include "input_waiter.h"

typedef void *(*ThreadFunction) (void *);

static slist<VideoCodec *> codecs;

void register_video_codec(VideoCodec *codec) {
    codecs.insert(codecs.end(),codec);
}

slist<VideoCodec *> get_video_codecs() {
    return codecs;
}

void VideoCodec::notifyProgress(double percent) {
    write(pipe_fd[1],&percent,sizeof(percent));
}

static void thread_fn(ThrData *d) {
    d->codec->doEncode(d->stream, d->fname, d->buffer);
}

void VideoCodec::receivedProgress(double percent) {
    progress=percent;
    notify("PROGRESS");
}

static void input_fn(VideoCodec *codec, int source) {
    double val;
    read(source,&val,sizeof(val));
    codec->receivedProgress(val);
}

void VideoCodec::startThread(ThrData *data_send) {
    pthread_create(&thread,NULL,(ThreadFunction)thread_fn, data_send);
}

void VideoCodec::encode(ImStream *stream, char *filename, Image *backbuffer) {
    ThrData *data_send=new ThrData();
    pipe(pipe_fd);
    data_send->codec=this;
    data_send->stream=stream;
    data_send->fname=strdup(filename);
    data_send->buffer=backbuffer;
    startThread(data_send);
    InputWaiter::Instance()->WaitRead(pipe_fd[0],(WaitFunc)input_fn, this);
}
