// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef __VIDEO_CODEC_H__
#define __VIDEO_CODEC_H__

#include <slist.h>
#include <imstream.h>
#include <pthread.h>

class VideoCodec;

class ThrData {
public:
    VideoCodec *codec;
    ImStream *stream;
    char *fname;
    Image *buffer;
};


class VideoCodec: public Subject {
private:
    int pipe_fd[2];
    pthread_t thread;
    void startThread(ThrData *data);
    double progress;
protected:
    void notifyProgress(double percent);
public:
    virtual ~VideoCodec(){}
    double getProgress() { return progress; }

    // Pseudo private method, neither redefine nor call
    void receivedProgress(double percent);

    // Pseudo-protected method, please, redefine it
    virtual void doEncode(ImStream *stream, char *filename, Image *backbuffer)=0;

    // Do not redefine this method!
    void encode(ImStream *stream, char *filename, Image *backbuffer);

    // Mandatory Information
    virtual char *getName()=0;
    virtual char *getVersion()=0;
    virtual char *getCopyright()=0;
    virtual char **getAuthors()=0;

    // Optional information
    virtual char *getDescription() { return "No description"; }

    // Config
    virtual bool configure() { return false; }
    virtual bool saveConfig(char *directory)=0;
};

/* This function allows modules to register their codecs */
extern void register_video_codec(VideoCodec *codec);

/* This function allows the program to see the registered codecs */
slist<VideoCodec *> get_video_codecs();

#endif
