// GAnSO - GNOME Animation StudiO
// Copyright (C) 2000 Ruben Lopez Gomez
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef __VIDEO_LOADER_H__
#define __VIDEO_LOADER_H__

#include <slist.h>
#include <string.h>
#include <prototype.h>
#include "videosource.h"

class VideoLoader: public VideoSource, public Prototype {
protected:
    char *filename;
public:
    VideoLoader() { filename=NULL; }
    void setFileName(const char *filename) {
	this->filename=strdup(filename);
        haveFileName();
    }
    virtual ~VideoLoader() { if (filename) free(filename); }
    char* getName() { return filename; }


    /* all loaders must implement this: */
    virtual bool canLoad(const char *filename)=0;
    virtual Image *read(int position)=0;
    virtual int size()=0;
    virtual char* getLoaderName()=0;
    virtual char* getLoaderVersion()=0;
    virtual char* getLoaderCopyright()=0;
    virtual char**getLoaderAuthors()=0;
    virtual Prototype *clone()=0;

    /* Optional */
    virtual char* getLoaderDescription() {return "No description";}
    virtual void haveFileName() {} // called when the filename is set
};

/* This function allows modules to register their loaders */
extern void register_video_loader(VideoLoader *loader);

/* This function allows the program to see the registered loaders */
slist<VideoLoader *> get_video_loaders();

VideoLoader *find_video_loader(char *filename);

#endif
